/**
 * 
 */
package cytoscape.PFPPipeline;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JOptionPane;

import cytoscape.CyEdge;
import cytoscape.CyNetwork;
import cytoscape.CyNode;
import cytoscape.Cytoscape;
import cytoscape.PFPPipeline.GOParser.SearchOntology;
import cytoscape.PFPPipeline.GOParser.term.GO_term;
import cytoscape.PFPPipeline.Interactions.InteractorProtein;
import cytoscape.data.CyAttributes;
import cytoscape.data.Semantics;
import cytoscape.layout.CyLayouts;

/**
 * @author hafeez
 * 
 */
public class BuildNetwork {

	/* Data members */

	//public CyNetwork cyNetwork;
	public CyNetwork cyInteractionNetwork;
	public CyNetwork cyInteractionScoreNetwork; // Network with interactors and their scores.
	public CyNetwork cyUpInteractionNetwork;
	public CyNetwork cyOnlyGONodesNetwork; // Network with only interactors(above threshold) and their GO Nodes.
	public CyNetwork cyGOAllNodesNetwork;	//Network with GO Node and under scoring nodes.
	public CyNetwork cyTargetTermsNetwork;
	public ArrayList<CyNode> nodes;
	public ArrayList<CyEdge> edges;
	public CyAttributes cyNodeAttrs, cyEdgeAttrs;
	public VisualizeNetwork vn;

	/**
	 * 
	 */
	public BuildNetwork() {
		// TODO Auto-generated constructor stub
		// Create visualization object to visualize the created network in
		// future.
		vn = new VisualizeNetwork();
	}

	@SuppressWarnings("unchecked")
	public void createNetwork() {
//			**********************************************
//			*******************STEP 01********************
//			**********************************************
		
		if (Pipeline.step.intValue() == 1) {
			if (Interactions.interactorProteins.size() > 0) {
				// Destroy previously created network.
				if (cyInteractionNetwork != null) {
					Cytoscape.destroyNetwork(cyInteractionNetwork);
				}
				// Create a network without a view and create first node for
				// query protein.
				cyInteractionNetwork = Cytoscape.createNetwork("Interaction Network",
						false);
				edges = new ArrayList<CyEdge>();
				nodes = new ArrayList<CyNode>();
				cyNodeAttrs = Cytoscape.getNodeAttributes();
				cyEdgeAttrs = Cytoscape.getEdgeAttributes();

				nodes.add(Cytoscape.getCyNode(Pipeline.query, true));
				// Add attributes
				
				if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
						nodes.get(0).getIdentifier(), "shownLabel",
						Pipeline.query);
				}else{
					cyNodeAttrs.setAttribute(
							nodes.get(0).getIdentifier(), "shownLabel",
							Interactions.queryProtein.getName());
				}
				
				// Add Attributes..
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
						"canonicalName", Interactions.queryProtein.getName());
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
						"Species", Interactions.queryProtein.getSpecies());
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
						"NSize", 0);
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(), "Type",
						"query");
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(), "colorValue",
						new Double(0.1));
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(), "colorValue",
								new Double(1.1));

				Iterator<InteractorProtein> uniqInteractorsIterator = Interactions.interactorProteins
						.iterator();
				InteractorProtein interactor;

				int i = 1; // Starting from node index 1.
				while (uniqInteractorsIterator.hasNext()) {
					interactor = uniqInteractorsIterator.next();

					// Add interactors (start will be 2nd node).
					nodes.add(Cytoscape.getCyNode(interactor.getUniprotID(), true));

					// Add attributes
					if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
							nodes.get(i).getIdentifier(), "shownLabel",
							interactor.getUniprotID());
					}else{
						cyNodeAttrs.setAttribute(
								nodes.get(i).getIdentifier(), "shownLabel",
								interactor.getName());
					}
					cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(),
							"canonicalName", interactor.getName());
					cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(),
							"Species", interactor.getSpecies());
					cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(),
							"Database", interactor.getDatabase());					
					cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(),
							"NSize", 0);
					cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(),
							"Type", "protein");
					cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(), "colorValue",
							new Double(0.1));
					// Add to network
					cyInteractionNetwork.addNode(nodes.get(i));
					// System.out.println("adding node" + interactor);
					
					// Add edges to nodes
					edges.add(Cytoscape.getCyEdge(nodes.get(0), nodes.get(i),
							Semantics.INTERACTION  , "pd", true));
					//Add Edge Attributes..
					cyEdgeAttrs.setAttribute(edges.get(i - 1).getIdentifier(),
							"InteractionType", interactor.getInteractionType());
					cyEdgeAttrs.setAttribute(edges.get(i - 1).getIdentifier(),
							"ExperimentCount", interactor.getExperimentCount());
					cyEdgeAttrs.setAttribute(edges.get(i - 1).getIdentifier(),
							"InteractionDetectionExperiment", interactor.getInteractionDectectionExp());
					cyEdgeAttrs.setAttribute(edges.get(i - 1).getIdentifier(),
							"edgeid", new Double(0.5)); 
					// For edges use i-1 because edges are 1 less than nodes.
					cyEdgeAttrs.setAttribute(edges.get(i - 1).getIdentifier(),
							"label", "PPI");

					cyInteractionNetwork.addEdge(edges.get(i - 1));

					i++;
				}
				//Cytoscape.createNetworkView(cyInteractionNetwork, "Direct/Associated Interaction Network!");
				// Apply Visual Style (to visualize the network)
				vn.ApplyVisualStyle(cyInteractionNetwork);
				// Create Network View.
				Cytoscape.createNetworkView(cyInteractionNetwork, "Direct/Associated Interaction Network!");
				// Maximize the view window		
				javax.swing.JInternalFrame f = 
						Cytoscape.getDesktop().getNetworkViewManager().getInternalFrame(Cytoscape.getCurrentNetworkView()); 
						try { 
						   f.setMaximum(true); 
						} 
						catch (Exception ex) 
						{}
				//Set the Layout
				CyLayouts.getLayout("kamada-kawai").doLayout();
				/*
				 * - force-directed - hierarchical - kamada-kawai -
				 * fruchterman-rheingold - degree-circle - attributes-layout -
				 * jgraph-annealing - jgraph-moen - jgraph-circle -
				 * jgraph-radial-tree - jgraph-gem - jgraph-spring -
				 * jgraph-sugiyama - jgraph-tree
				 */

			}
		}else if (Pipeline.step.intValue() == 2) {
			DecimalFormat dFormat = new DecimalFormat("#.####");
			if (Interactions.interactorProteins.size() > 0) {
				// Destroy previously created network.
				if (cyInteractionScoreNetwork != null) {
					Cytoscape.destroyNetwork(cyInteractionScoreNetwork);
				}
				// Create a network without a view and create first node for
				// query protein.
				cyInteractionScoreNetwork = Cytoscape.createNetwork("Interaction Network with PIS",
						false);
				edges = new ArrayList<CyEdge>();
				nodes = new ArrayList<CyNode>();
				cyNodeAttrs = Cytoscape.getNodeAttributes();
				cyEdgeAttrs = Cytoscape.getEdgeAttributes();

				nodes.add(Cytoscape.getCyNode(Pipeline.query, true));
				// Add attributes
				if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
						nodes.get(0).getIdentifier(), "shownLabel",
						Pipeline.query);
				}else{
					cyNodeAttrs.setAttribute(
							nodes.get(0).getIdentifier(), "shownLabel",
							Interactions.queryProtein.getName());
				}
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
						"canonicalName", Interactions.queryProtein.getName());
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
						"Species", Interactions.queryProtein.getSpecies());
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
						"NSize", 0);
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(), "Type",
						"query");
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(), "colorValue",
						new Double(1.1));

				Iterator<InteractorProtein> uniqInteractorsIterator = Interactions.interactorProteins
						.iterator();
				InteractorProtein interactor;

				int i = 1; // Starting from node index 1.
				while (uniqInteractorsIterator.hasNext()) {
					interactor = uniqInteractorsIterator.next();

					// Add interactors (start will be 2nd node).
					nodes.add(Cytoscape.getCyNode(interactor.getUniprotID(), true));

					// Add attributes
					if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
							nodes.get(i).getIdentifier(), "shownLabel",
							interactor.getUniprotID());
					}else{
						cyNodeAttrs.setAttribute(
								nodes.get(i).getIdentifier(), "shownLabel",
								interactor.getName());
					}
					cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(),
							"canonicalName", interactor.getName());
					cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(),
							"Species", interactor.getSpecies());
					cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(),
							"Database", interactor.getDatabase());
					cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(),
							"PIS", interactor.getPISValue());					
					cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(),
							"NSize", 0);
					cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(),
							"Type", "protein");
					if(interactor.getPISValue()>=Pipeline.simThreshold){
					cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(), "colorValue",
							new Double(1.1));}
					else{
						cyNodeAttrs.setAttribute(nodes.get(i).getIdentifier(), "colorValue",
								new Double(0.1));
					}
					// Add to network
					cyInteractionScoreNetwork.addNode(nodes.get(i));
					// System.out.println("adding node" + interactor);
					// Add edges to nodes
					edges.add(Cytoscape.getCyEdge(nodes.get(0), nodes.get(i),
							Semantics.INTERACTION, "pp", true));
					//Add Edge Attributes..
					cyEdgeAttrs.setAttribute(edges.get(i - 1).getIdentifier(),
							"InteractionType", interactor.getInteractionType());
					cyEdgeAttrs.setAttribute(edges.get(i - 1).getIdentifier(),
							"ExperimentCount", interactor.getExperimentCount());
					cyEdgeAttrs.setAttribute(edges.get(i - 1).getIdentifier(),
							"InteractionDetectionExperiment", interactor.getInteractionDectectionExp());
					
					cyEdgeAttrs.setAttribute(edges.get(i - 1).getIdentifier(),
							"edgeid", new Double(0.5)); // For edges use i-1
														// because edges are
					// 1 less than nodes.
					cyEdgeAttrs.setAttribute(edges.get(i - 1).getIdentifier(),
							"label","PIS: \n"+ dFormat.format(Pipeline.PIScore[i-1]).toString());
					

					cyInteractionScoreNetwork.addEdge(edges.get(i - 1));

					i++;
				}
				//Cytoscape.createNetworkView(cyInteractionNetwork, "Direct/Associated Interaction Network!");
				// Apply Visual Style (to visualize the network)
				vn.ApplyVisualStyle(cyInteractionScoreNetwork);
				// Create Network View.
				Cytoscape.createNetworkView(cyInteractionScoreNetwork, "Direct/Associated Interaction Network!");
				// Maximize the view window		
				javax.swing.JInternalFrame f = 
						Cytoscape.getDesktop().getNetworkViewManager().getInternalFrame(Cytoscape.getCurrentNetworkView()); 
						try { 
						   f.setMaximum(true); 
						} 
						catch (Exception ex) 
						{}
				//Set the Layout
				CyLayouts.getLayout("kamada-kawai").doLayout();
				/*
				 * - force-directed - hierarchical - kamada-kawai -
				 * fruchterman-rheingold - degree-circle - attributes-layout -
				 * jgraph-annealing - jgraph-moen - jgraph-circle -
				 * jgraph-radial-tree - jgraph-gem - jgraph-spring -
				 * jgraph-sugiyama - jgraph-tree
				 */

			}
		}

		else if (Pipeline.step.intValue() == 3) {

			// Destroy previously created network.
			if (cyGOAllNodesNetwork != null) {
				Cytoscape.destroyNetwork(cyGOAllNodesNetwork);
				Cytoscape.destroyNetwork(cyOnlyGONodesNetwork);
				Cytoscape.destroyNetwork(cyUpInteractionNetwork);
			}
			// We are in the 2nd step of PFP-pipeline.
			if (Pipeline.filteredInteractors.size() > 0) {
				// Create a network without a view and create first node for
				// query protein.
				DecimalFormat dFormat = new DecimalFormat("#.####");
				cyGOAllNodesNetwork = Cytoscape.createNetwork("All Interactions Network with GO",
						false);
				cyOnlyGONodesNetwork = Cytoscape.createNetwork("Filtered Interaction Network with GO",
						false);
				cyUpInteractionNetwork = Cytoscape.createNetwork("Filtered Interaction Network without GO",
						false);
				edges = new ArrayList<CyEdge>();
				nodes = new ArrayList<CyNode>();
				cyNodeAttrs = Cytoscape.getNodeAttributes();
				cyEdgeAttrs = Cytoscape.getEdgeAttributes();

				nodes.add(Cytoscape.getCyNode(Pipeline.query, true));
				// Add attributes
				if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
						nodes.get(0).getIdentifier(), "shownLabel",
						Pipeline.query);
				}else{
					cyNodeAttrs.setAttribute(
							nodes.get(0).getIdentifier(), "shownLabel",
							Interactions.queryProtein.getName());
				}
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
						"canonicalName", Interactions.queryProtein.getName());
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
						"Species", Interactions.queryProtein.getSpecies());
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
						"NSize", 0);
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(), "Type",
						"query");
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(), "colorValue",
						new Double(1.1));

				Iterator<InteractorProtein> filteredInteractorsIterator = Pipeline.filteredInteractors
						.iterator();

				// int i = 1; // Starting from node index 1.
				int nodeIndex, edgeIndex, interactorIndex = 0, goNodes = 0;
				ArrayList<String> singlePath;
				String goNode;
				InteractorProtein interactor;
//				System.out.println("Filtered Interactors size: " +Pipeline.filteredInteractors.size());
				while (filteredInteractorsIterator.hasNext()) {
					interactor = filteredInteractorsIterator.next();

					// Add interactors (start will be 2nd node).
					CyNode node = Cytoscape.getCyNode(interactor.getUniprotID(), true);
					nodes.add(node);
					nodeIndex = nodes.indexOf(node);
					// System.out.println(nodeIndex);

					// Add attributes
					if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
							nodes.get(nodeIndex).getIdentifier(), "shownLabel",
							interactor.getUniprotID());
					}else{
						cyNodeAttrs.setAttribute(
								nodes.get(nodeIndex).getIdentifier(), "shownLabel",
								interactor.getName());
					}
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
							"canonicalName", interactor.getName());
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
							"Species", interactor.getSpecies());
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
							"Database", interactor.getDatabase());
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
							"PIS", interactor.getPISValue());
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex)
							.getIdentifier(), "NSize", 0);
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex)
							.getIdentifier(), "Type", "protein");
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(), "colorValue",
							new Double(1.1));
					// Add to network
					cyGOAllNodesNetwork.addNode(nodes.get(nodeIndex));
					cyOnlyGONodesNetwork.addNode(nodes.get(nodeIndex));
					cyUpInteractionNetwork.addNode(nodes.get(nodeIndex));
					// System.out.println("adding node" + interactor);

					// Check annotations of this node (i-1). If it has
					// annotations add annotation paths to network.

					if (Pipeline.goAnnotations[interactorIndex].size() > 0) {
						int nodeIndex2, edgeIndex2,prevNodeIndex;
						CyNode node2=Cytoscape.getCyNode(
								"initialize", true);;

						for (int i = 0; i < SearchOntology.interactorPaths.get(
								interactorIndex).size(); i++) {
							singlePath = (ArrayList<String>) SearchOntology.interactorPaths
									.get(interactorIndex).get(i);
							//System.out.print("\n");

							for (int j = singlePath.size() - 1; j >= 0; j--) {
								goNode = singlePath.get(j);
								
								if (goNode.contains("0003674")) {
									node2 = Cytoscape.getCyNode(
											goNodes + "", true);
									nodes.add(node2);
									nodeIndex2 = nodes.indexOf(node2);
									// System.out.println(nodeIndex);

									// Add attributes
									if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
											nodes.get(nodeIndex2).getIdentifier(), "shownLabel",
											"GO:"+goNode);
									}else{
										cyNodeAttrs.setAttribute(
												nodes.get(nodeIndex2).getIdentifier(), "shownLabel",
												SearchOntology.id2Name(goNode));
									}
									
//									System.out.println(SearchOntology.id2Name(goNode));
									cyNodeAttrs.setAttribute(nodes.get(nodeIndex2).getIdentifier(),
											"GOID", "GO:"+goNode);
									cyNodeAttrs.setAttribute(nodes.get(nodeIndex2).getIdentifier(),
											"canonicalName", SearchOntology.id2Name(goNode));
									cyNodeAttrs.setAttribute(
											nodes.get(nodeIndex2)
													.getIdentifier(), "NSize",
													2);
									cyNodeAttrs.setAttribute(
											nodes.get(nodeIndex2)
													.getIdentifier(), "Type",
											"GO");
									cyNodeAttrs.setAttribute(nodes.get(nodeIndex2)
											.getIdentifier(), "colorValue",
											new Double(2.1));
									// Add to network
									cyGOAllNodesNetwork.addNode(nodes.get(nodeIndex2));
									cyOnlyGONodesNetwork.addNode(nodes.get(nodeIndex2));

									// Add edge
									CyEdge edge2;
									edge2 = Cytoscape.getCyEdge(
											nodes.get(nodeIndex),
											nodes.get(nodeIndex2),
											Semantics.INTERACTION, "MF",
											true);
									edges.add(edge2);
									edgeIndex2 = edges.indexOf(edge2);
									cyEdgeAttrs.setAttribute(
											edges.get(edgeIndex2)
													.getIdentifier(), "edgeid",
											0.5); // For edges use i-1 because
									// edges are
									// 1 less than nodes.
									cyEdgeAttrs.setAttribute(
											edges.get(edgeIndex2)
													.getIdentifier(), "label",
											"GO \n");

									cyGOAllNodesNetwork.addEdge(edges.get(edgeIndex2));
									cyOnlyGONodesNetwork.addEdge(edges.get(edgeIndex2));
								}
								else{
									prevNodeIndex=nodes.indexOf(node2);
									node2 = Cytoscape.getCyNode(
											goNodes + "", true);
									nodes.add(node2);
									nodeIndex2 = nodes.indexOf(node2);
									// System.out.println(nodeIndex);

									// Add attributes
									if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
											nodes.get(nodeIndex2).getIdentifier(), "shownLabel",
											"GO:"+goNode);
									}else{
										cyNodeAttrs.setAttribute(
												nodes.get(nodeIndex2).getIdentifier(), "shownLabel",
												SearchOntology.id2Name(goNode));
									}
									cyNodeAttrs.setAttribute(nodes.get(nodeIndex2).getIdentifier(),
											"GOID", "GO:"+goNode);
									cyNodeAttrs.setAttribute(nodes.get(nodeIndex2).getIdentifier(),
											"canonicalName", SearchOntology.id2Name(goNode));
									cyNodeAttrs.setAttribute(
											nodes.get(nodeIndex2)
													.getIdentifier(), "NSize",
													2);
									cyNodeAttrs.setAttribute(
											nodes.get(nodeIndex2)
													.getIdentifier(), "Type",
											"GO");
									cyNodeAttrs.setAttribute(nodes.get(nodeIndex2)
											.getIdentifier(), "colorValue",
											new Double(2.1));
									//System.out.println("Id: "+goNode+"Name: "+SearchOntology.id2Name(goNode));
									// Add to network
									cyGOAllNodesNetwork.addNode(nodes.get(nodeIndex2));
									cyOnlyGONodesNetwork.addNode(nodes.get(nodeIndex2));

									// Add edge
									CyEdge edge2;
									edge2 = Cytoscape.getCyEdge(
											nodes.get(prevNodeIndex),
											nodes.get(nodeIndex2),
											Semantics.INTERACTION, "GO",
											true);
									edges.add(edge2);
									edgeIndex2 = edges.indexOf(edge2);
									cyEdgeAttrs.setAttribute(
											edges.get(edgeIndex2)
													.getIdentifier(), "edgeid",
											0.5); // For edges use i-1 because
									// edges are
									// 1 less than nodes.
									cyEdgeAttrs.setAttribute(
											edges.get(edgeIndex2)
													.getIdentifier(), "label",
											"IS_A \n");

									cyGOAllNodesNetwork.addEdge(edges.get(edgeIndex2));
									cyOnlyGONodesNetwork.addEdge(edges.get(edgeIndex2));
									
								}
								goNodes++;
								//System.out.print(goNode + " ");
							}
						}

					}

					// ************ Add edges to nodes **************
					CyEdge edge;
					edge = Cytoscape.getCyEdge(nodes.get(0),
							nodes.get(nodeIndex), Semantics.INTERACTION, "pp",
							true);
					edges.add(edge);
					edgeIndex = edges.indexOf(edge);
					
					cyEdgeAttrs.setAttribute(edges.get(edgeIndex).getIdentifier(),
							"InteractionType", interactor.getInteractionType());
					cyEdgeAttrs.setAttribute(edges.get(edgeIndex).getIdentifier(),
							"ExperimentCount", interactor.getExperimentCount());
					cyEdgeAttrs.setAttribute(edges.get(edgeIndex).getIdentifier(),
							"InteractionDetectionExperiment", interactor.getInteractionDectectionExp());
					
					
					cyEdgeAttrs.setAttribute(edges.get(edgeIndex)
							.getIdentifier(), "edgeid", Pipeline.upPIScore
							.get(interactorIndex)); 
					
					// 1 less than nodes.
					cyEdgeAttrs.setAttribute(
							edges.get(edgeIndex).getIdentifier(),
							"label",
							"PIS: \n"
									+ dFormat.format(
											Pipeline.upPIScore
													.get(interactorIndex))
											.toString());

					cyGOAllNodesNetwork.addEdge(edges.get(edgeIndex));
					cyOnlyGONodesNetwork.addEdge(edges.get(edgeIndex));
					cyUpInteractionNetwork.addEdge(edges.get(edgeIndex));

					interactorIndex++;
				}
				// For Complete Network				
				if(Pipeline.filteredInteractors.size() < Interactions.interactorProteins.size()){
					Iterator<InteractorProtein> uniqInteractorsIterator = Interactions.interactorProteins
							.iterator();					
					int k = 1; // Starting from node index 1.
					while (uniqInteractorsIterator.hasNext()) {
						InteractorProtein uScoreInteractor=uniqInteractorsIterator.next();
						if(!Pipeline.filteredInteractors.contains(uScoreInteractor)){
							
							// Add interactors (start will be 2nd node).
							CyNode node = Cytoscape.getCyNode(uScoreInteractor.getUniprotID(), true);
							nodes.add(node);
							nodeIndex = nodes.indexOf(node);
							// System.out.println(nodeIndex);

							// Add attributes
							if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
									nodes.get(nodeIndex).getIdentifier(), "shownLabel",
									uScoreInteractor.getUniprotID());
							}else{
								cyNodeAttrs.setAttribute(
										nodes.get(nodeIndex).getIdentifier(), "shownLabel",
										uScoreInteractor.getName());
							}
							cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
									"canonicalName", uScoreInteractor.getName());
							cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
									"Species", uScoreInteractor.getSpecies());
							cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
									"Database", uScoreInteractor.getDatabase());
							cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
									"PIS", uScoreInteractor.getPISValue());
							cyNodeAttrs.setAttribute(nodes.get(nodeIndex)
									.getIdentifier(), "NSize", 0);
							cyNodeAttrs.setAttribute(nodes.get(nodeIndex)
									.getIdentifier(), "Type", "protein");							
							cyNodeAttrs.setAttribute(nodes.get(nodeIndex)
									.getIdentifier(), "colorValue",
									new Double(0.1));
							// Add to network
							cyGOAllNodesNetwork.addNode(nodes.get(nodeIndex));
							
							// ************ Add edges to nodes **************
							CyEdge edge;
							edge = Cytoscape.getCyEdge(nodes.get(0),
									nodes.get(nodeIndex), Semantics.INTERACTION, "pp",
									true);
							edges.add(edge);
							edgeIndex = edges.indexOf(edge);
							cyEdgeAttrs.setAttribute(edges.get(edgeIndex).getIdentifier(),
									"InteractionType", uScoreInteractor.getInteractionType());
							cyEdgeAttrs.setAttribute(edges.get(edgeIndex).getIdentifier(),
									"ExperimentCount", uScoreInteractor.getExperimentCount());
							cyEdgeAttrs.setAttribute(edges.get(edgeIndex).getIdentifier(),
									"InteractionDetectionExperiment", uScoreInteractor.getInteractionDectectionExp());
							
							cyEdgeAttrs.setAttribute(edges.get(edgeIndex)
									.getIdentifier(), "edgeid", Pipeline.PIScore[k-1]); // For edges use i-1 because
															
							cyEdgeAttrs.setAttribute(
									edges.get(edgeIndex).getIdentifier(),
									"label",
									"PIS: \n"
											+ dFormat.format(
													Pipeline.PIScore[k-1])
													.toString());

							cyGOAllNodesNetwork.addEdge(edges.get(edgeIndex));
							
						}
						k++;
						
					}
				}
				// Apply Visual Style (to visualize the network)
				vn.ApplyVisualStyle(cyGOAllNodesNetwork);
				Cytoscape.createNetworkView(cyGOAllNodesNetwork,
						"Direct/Associated Interaction Network!");
				// Maximize the view window
				javax.swing.JInternalFrame f = Cytoscape.getDesktop()
						.getNetworkViewManager()
						.getInternalFrame(Cytoscape.getCurrentNetworkView());
				try {
					f.setMaximum(true);
				} catch (Exception ex) {
				}
				// CyLayouts.getLayout("jgraph-tree").doLayout();
				CyLayouts.getLayout("jgraph-tree").doLayout();

			}
			
		}
		else if (Pipeline.step.intValue() == 4){

			// Destroy previously created network.
			if (cyOnlyGONodesNetwork != null) {
//				Cytoscape.destroyNetwork(cyGOAllNodesNetwork);
				Cytoscape.destroyNetwork(cyOnlyGONodesNetwork);				
			}
			// We are in the 4th step of PFP-pipeline.
			if (Pipeline.filteredInteractors.size() > 0) {
				// Create a network without a view and create first node for
				// query protein.
				DecimalFormat dFormat = new DecimalFormat("#.####");				
				cyOnlyGONodesNetwork = Cytoscape.createNetwork("Filtered Interaction Network with GO",
						false);
				edges = new ArrayList<CyEdge>();
				nodes = new ArrayList<CyNode>();
				cyNodeAttrs = Cytoscape.getNodeAttributes();
				cyEdgeAttrs = Cytoscape.getEdgeAttributes();

				nodes.add(Cytoscape.getCyNode(Pipeline.query, true));
				// Add attributes
				if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
						nodes.get(0).getIdentifier(), "shownLabel",
						Pipeline.query);
				}else{
					cyNodeAttrs.setAttribute(
							nodes.get(0).getIdentifier(), "shownLabel",
							Interactions.queryProtein.getName());
				}
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
						"canonicalName", Interactions.queryProtein.getName());
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
						"Species", Interactions.queryProtein.getSpecies());
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
						"NSize", 0);
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(), "Type",
						"query");
				cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(), "colorValue",
						new Double(1.1));

				Iterator<InteractorProtein> filteredInteractorsIterator = Pipeline.filteredInteractors
						.iterator();

				// int i = 1; // Starting from node index 1.
				int nodeIndex, edgeIndex, interactorIndex = 0, goNodes = 0;
				ArrayList<String> singlePath;
				String goNode;
				InteractorProtein interactor;
//				System.out.println("Filtered Interactors size: " +Pipeline.filteredInteractors.size());
				while (filteredInteractorsIterator.hasNext()) {
					interactor = filteredInteractorsIterator.next();

					// Add interactors (start will be 2nd node).
					CyNode node = Cytoscape.getCyNode(interactor.getUniprotID(), true);
					nodes.add(node);
					nodeIndex = nodes.indexOf(node);
					// System.out.println(nodeIndex);

					// Add attributes
					if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
							nodes.get(nodeIndex).getIdentifier(), "shownLabel",
							interactor.getUniprotID());
					}else{
						cyNodeAttrs.setAttribute(
								nodes.get(nodeIndex).getIdentifier(), "shownLabel",
								interactor.getName());
					}
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
							"canonicalName", interactor.getName());
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
							"Species", interactor.getSpecies());
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
							"Database", interactor.getDatabase());
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
							"PIS", interactor.getPISValue());
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex)
							.getIdentifier(), "NSize", 0);
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex)
							.getIdentifier(), "Type", "protein");
					cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(), "colorValue",
							new Double(1.1));
					// Add to network
//					cyGOAllNodesNetwork.addNode(nodes.get(nodeIndex));
					cyOnlyGONodesNetwork.addNode(nodes.get(nodeIndex));
//					cyUpInteractionNetwork.addNode(nodes.get(nodeIndex));
					// System.out.println("adding node" + interactor);

					// Check annotations of this node (i-1). If it has
					// annotations add annotation paths to network.

					if (Pipeline.goAnnotations[interactorIndex].size() > 0) {
						int nodeIndex2, edgeIndex2,prevNodeIndex;
						CyNode node2=Cytoscape.getCyNode(
								"initialize", true);;

						for (int i = 0; i < SearchOntology.interactorPaths.get(
								interactorIndex).size(); i++) {
							singlePath = (ArrayList<String>) SearchOntology.interactorPaths
									.get(interactorIndex).get(i);
							//System.out.print("\n");

							for (int j = singlePath.size() - 1; j >= 0; j--) {
								goNode = singlePath.get(j);
								
								if (goNode.contains("0003674")) {
									node2 = Cytoscape.getCyNode(
											goNodes + "", true);
									nodes.add(node2);
									nodeIndex2 = nodes.indexOf(node2);
									// System.out.println(nodeIndex);

									// Add attributes
									if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
											nodes.get(nodeIndex2).getIdentifier(), "shownLabel",
											"GO:"+goNode);
									}else{
										cyNodeAttrs.setAttribute(
												nodes.get(nodeIndex2).getIdentifier(), "shownLabel",
												SearchOntology.id2Name(goNode));
									}
									cyNodeAttrs.setAttribute(nodes.get(nodeIndex2).getIdentifier(),
											"GOID", "GO:"+goNode);
									cyNodeAttrs.setAttribute(nodes.get(nodeIndex2).getIdentifier(),
											"canonicalName", SearchOntology.id2Name(goNode));
//									System.out.println(SearchOntology.id2Name(goNode));
									cyNodeAttrs.setAttribute(
											nodes.get(nodeIndex2)
													.getIdentifier(), "NSize",
													2);
									cyNodeAttrs.setAttribute(
											nodes.get(nodeIndex2)
													.getIdentifier(), "Type",
											"GO");
									if (Pipeline.upInteractorPaths
											.contains(singlePath)) {
										cyNodeAttrs.setAttribute(
												nodes.get(nodeIndex2)
														.getIdentifier(),
												"colorValue", new Double(4.1));
									} else {
										cyNodeAttrs.setAttribute(
												nodes.get(nodeIndex2)
														.getIdentifier(),
												"colorValue", new Double(2.1));
									}
									// Add to network
//									cyGOAllNodesNetwork.addNode(nodes.get(nodeIndex2));
									cyOnlyGONodesNetwork.addNode(nodes.get(nodeIndex2));

									// Add edge
									CyEdge edge2;
									edge2 = Cytoscape.getCyEdge(
											nodes.get(nodeIndex),
											nodes.get(nodeIndex2),
											Semantics.INTERACTION, "MF",
											true);
									edges.add(edge2);
									edgeIndex2 = edges.indexOf(edge2);
									cyEdgeAttrs.setAttribute(
											edges.get(edgeIndex2)
													.getIdentifier(), "edgeid",
											0.5); // For edges use i-1 because
									// edges are
									// 1 less than nodes.
									cyEdgeAttrs.setAttribute(
											edges.get(edgeIndex2)
													.getIdentifier(), "label",
											"GO \n");

//									cyGOAllNodesNetwork.addEdge(edges.get(edgeIndex2));
									cyOnlyGONodesNetwork.addEdge(edges.get(edgeIndex2));
								}
								else{
									prevNodeIndex=nodes.indexOf(node2);
									node2 = Cytoscape.getCyNode(
											goNodes + "", true);
									nodes.add(node2);
									nodeIndex2 = nodes.indexOf(node2);
									// System.out.println(nodeIndex);

									// Add attributes
									if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
											nodes.get(nodeIndex2).getIdentifier(), "shownLabel",
											"GO:"+goNode);
									}else{
										cyNodeAttrs.setAttribute(
												nodes.get(nodeIndex2).getIdentifier(), "shownLabel",
												SearchOntology.id2Name(goNode));
									}
									cyNodeAttrs.setAttribute(nodes.get(nodeIndex2).getIdentifier(),
											"GOID", "GO:"+goNode);
									cyNodeAttrs.setAttribute(nodes.get(nodeIndex2).getIdentifier(),
											"canonicalName", SearchOntology.id2Name(goNode));
									cyNodeAttrs.setAttribute(
											nodes.get(nodeIndex2)
													.getIdentifier(), "NSize",
													2);
									cyNodeAttrs.setAttribute(
											nodes.get(nodeIndex2)
													.getIdentifier(), "Type",
											"GO");
									if (Pipeline.upInteractorPaths
											.contains(singlePath)) {
										cyNodeAttrs.setAttribute(
												nodes.get(nodeIndex2)
														.getIdentifier(),
												"colorValue", new Double(4.1));
									} else {
										cyNodeAttrs.setAttribute(
												nodes.get(nodeIndex2)
														.getIdentifier(),
												"colorValue", new Double(2.1));
									}
//									System.out.println("Id: "+goNode+"Name: "+SearchOntology.id2Name(goNode));
									// Add to network
//									cyGOAllNodesNetwork.addNode(nodes.get(nodeIndex2));
									cyOnlyGONodesNetwork.addNode(nodes.get(nodeIndex2));

									// Add edge
									CyEdge edge2;
									edge2 = Cytoscape.getCyEdge(
											nodes.get(prevNodeIndex),
											nodes.get(nodeIndex2),
											Semantics.INTERACTION, "GO",
											true);
									edges.add(edge2);
									edgeIndex2 = edges.indexOf(edge2);
									cyEdgeAttrs.setAttribute(
											edges.get(edgeIndex2)
													.getIdentifier(), "edgeid",
											0.5); // For edges use i-1 because
									// edges are
									// 1 less than nodes.
									cyEdgeAttrs.setAttribute(
											edges.get(edgeIndex2)
													.getIdentifier(), "label",
											"IS_A \n");

//									cyGOAllNodesNetwork.addEdge(edges.get(edgeIndex2));
									cyOnlyGONodesNetwork.addEdge(edges.get(edgeIndex2));
									
								}
								goNodes++;
								//System.out.print(goNode + " ");
							}
						}

					}

					// ************ Add edges to nodes **************
					CyEdge edge;
					edge = Cytoscape.getCyEdge(nodes.get(0),
							nodes.get(nodeIndex), Semantics.INTERACTION, "pp",
							true);
					edges.add(edge);
					edgeIndex = edges.indexOf(edge);
					cyEdgeAttrs.setAttribute(edges.get(edgeIndex).getIdentifier(),
							"InteractionType", interactor.getInteractionType());
					cyEdgeAttrs.setAttribute(edges.get(edgeIndex).getIdentifier(),
							"ExperimentCount", interactor.getExperimentCount());
					cyEdgeAttrs.setAttribute(edges.get(edgeIndex).getIdentifier(),
							"InteractionDetectionExperiment", interactor.getInteractionDectectionExp());
					
					cyEdgeAttrs.setAttribute(edges.get(edgeIndex)
							.getIdentifier(), "edgeid", Pipeline.upPIScore
							.get(interactorIndex)); // For edges use i-1 because
													// edges are
					// 1 less than nodes.
					cyEdgeAttrs.setAttribute(
							edges.get(edgeIndex).getIdentifier(),
							"label",
							"PIS: \n"
									+ dFormat.format(
											Pipeline.upPIScore
													.get(interactorIndex))
											.toString());

//					cyGOAllNodesNetwork.addEdge(edges.get(edgeIndex));
					cyOnlyGONodesNetwork.addEdge(edges.get(edgeIndex));
//					cyUpInteractionNetwork.addEdge(edges.get(edgeIndex));

					interactorIndex++;
				}			
				
				// Apply Visual Style (to visualize the network)
				vn.ApplyVisualStyle(cyOnlyGONodesNetwork);
				Cytoscape.createNetworkView(cyOnlyGONodesNetwork,
						"Direct/Associated Interaction Network!");
				// Maximize the view window
				javax.swing.JInternalFrame f = Cytoscape.getDesktop()
						.getNetworkViewManager()
						.getInternalFrame(Cytoscape.getCurrentNetworkView());
				try {
					f.setMaximum(true);
				} catch (Exception ex) {
				}
				// CyLayouts.getLayout("jgraph-tree").doLayout();
				CyLayouts.getLayout("jgraph-tree").doLayout();

			}			
		}else if(Pipeline.step.intValue()==5){
			/* Shows Unique list of Functions */	

			// Destroy previously created network.
		if (cyTargetTermsNetwork != null) {
				Cytoscape.destroyNetwork(cyTargetTermsNetwork);
			}
			// Create Network.
			cyTargetTermsNetwork=Cytoscape.createNetwork("Target Gene Ontology Terms Network!", false);
			edges = new ArrayList<CyEdge>();
			nodes = new ArrayList<CyNode>();
			cyNodeAttrs = Cytoscape.getNodeAttributes();
			cyEdgeAttrs = Cytoscape.getEdgeAttributes();

			nodes.add(Cytoscape.getCyNode(Pipeline.query, true));
			// Add attributes
			if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
					nodes.get(0).getIdentifier(), "shownLabel",
					Pipeline.query);
			}else{
				cyNodeAttrs.setAttribute(
						nodes.get(0).getIdentifier(), "shownLabel",
						Interactions.queryProtein.getName());
			}
			cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
					"canonicalName", Interactions.queryProtein.getName());
			cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
					"Species", Interactions.queryProtein.getSpecies());
			cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(),
					"NSize", 0);
			cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(), "Type",
					"query");
			cyNodeAttrs.setAttribute(nodes.get(0).getIdentifier(), "colorValue",
					new Double(1.1));
			

			/*
			 * 
			 * If more than one annotation then create netowrk from target annotation set.					 * 
			 *
			 */

			// Create a network and create first node for query protein.
			DecimalFormat dFormat = new DecimalFormat("#.####");
									
			String annotation;
			int nodeIndex,edgeIndex;
			for(int i=0;i<Pipeline.targetAnnotations.size();i++){
				annotation = Pipeline.targetAnnotations.get(i);
				
//				System.out.println("Creating network for: "+annotation);

				// Add interactors (start will be 2nd node).
				CyNode node = Cytoscape.getCyNode("GO:"+annotation, true);
				nodes.add(node);
				nodeIndex = nodes.indexOf(node);
				// System.out.println(nodeIndex);

				// Add attributes
				if(Pipeline.showNamesFlag==false){cyNodeAttrs.setAttribute(
						nodes.get(nodeIndex).getIdentifier(), "shownLabel",
						"GO:"+annotation);
				}else{
					cyNodeAttrs.setAttribute(
							nodes.get(nodeIndex).getIdentifier(), "shownLabel",
							SearchOntology.id2Name(annotation));
				}
				cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
						"GOID", "GO:"+annotation);
				cyNodeAttrs.setAttribute(nodes.get(nodeIndex).getIdentifier(),
						"canonicalName", SearchOntology.id2Name(annotation));
				
				cyNodeAttrs.setAttribute(
						nodes.get(nodeIndex)
								.getIdentifier(), "NSize",
								2);
				cyNodeAttrs.setAttribute(
						nodes.get(nodeIndex)
								.getIdentifier(), "Type",
						"GO");
				cyNodeAttrs.setAttribute(
						nodes.get(nodeIndex)
								.getIdentifier(),
						"colorValue", new Double(4.1));	
				
				// Add to network
				cyTargetTermsNetwork.addNode(nodes.get(nodeIndex));
				
				// ************ Add edges to nodes **************
				CyEdge edge;
				edge = Cytoscape.getCyEdge(nodes.get(0),
						nodes.get(nodeIndex), Semantics.INTERACTION, "annotation",
						true);
				edges.add(edge);
				edgeIndex = edges.indexOf(edge);
				cyEdgeAttrs.setAttribute(edges.get(edgeIndex)
						.getIdentifier(), "edgeid", 0.5); // For edges use i-1 because
				if(Pipeline.filteredInteractors.size()>1){								
				cyEdgeAttrs.setAttribute(
						edges.get(edgeIndex).getIdentifier(),
						"label",
						"GOSS:\n"+dFormat.format(Pipeline.targetAnnotationsSim.get(i)).toString());
				}else{
					cyEdgeAttrs.setAttribute(
							edges.get(edgeIndex).getIdentifier(),
							"label",
							"has\nfunction");		
					
				}

				cyTargetTermsNetwork.addEdge(edges.get(edgeIndex));
				
			}
			
			// Apply Visual Style (to visualize the network)
			vn.ApplyVisualStyle(cyTargetTermsNetwork);
			
			Cytoscape.createNetworkView(cyTargetTermsNetwork,
					"Target Gene Ontology Terms Network!");
			
			javax.swing.JInternalFrame f = 
					Cytoscape.getDesktop().getNetworkViewManager().getInternalFrame(Cytoscape.getCurrentNetworkView()); 
					try { 
					   f.setMaximum(true); 
					} 
					catch (Exception ex) 
					{} 
					
			CyLayouts.getLayout("jgraph-sugiyama").doLayout();
			/*
			 * - force-directed - hierarchical - kamada-kawai -
			 * fruchterman-rheingold - degree-circle - attributes-layout -
			 * jgraph-annealing - jgraph-moen - jgraph-circle -
			 * jgraph-radial-tree - jgraph-gem - jgraph-spring -
			 * jgraph-sugiyama - jgraph-tree
			 */				
					
		}			

	}
	

/* Show Filtered Network */

	public void showFileredNetwork(CyNetwork network){
//		System.out.println("Network Name: "+network.getTitle()+"\nNode count: "+network.getNodeCount());
		Cytoscape.destroyNetworkView(Cytoscape.getCurrentNetworkView());		
		Cytoscape.destroyNetworkView(network);
		// Apply Visual Style (to visualize the network)
//		System.out.println("building network...."+network.getIdentifier());
//		network.getNodeCount();
//		CyNode node;
//		node=(CyNode) network.getNode(0);
//		node.getIdentifier().
		List<CyNode> nodeList= network.nodesList();
		for(CyNode node: nodeList){
			if(Pipeline.showNamesFlag==true){
				String label=cyNodeAttrs.getAttribute(node.getIdentifier(), "canonicalName").toString();
				cyNodeAttrs.setAttribute(node.getIdentifier(),"shownLabel",label);				
			}else{
				String id;
				if(cyNodeAttrs.getAttribute(node.getIdentifier(), "Type").equals("GO")){
					id=cyNodeAttrs.getAttribute(node.getIdentifier(), "GOID").toString();
				}else{
					id=node.getIdentifier().toString();
				}
				cyNodeAttrs.setAttribute(node.getIdentifier(),"shownLabel",id);				
			}			
		}
		vn.ApplyVisualStyle(network);
		Cytoscape.createNetworkView(network,
				"Direct/Associated Interaction Network!");
		// Maximize the view window
		javax.swing.JInternalFrame f = Cytoscape.getDesktop()
				.getNetworkViewManager()
				.getInternalFrame(Cytoscape.getCurrentNetworkView());
		try {
			f.setMaximum(true);
		} catch (Exception ex) {
		}
		// CyLayouts.getLayout("jgraph-tree").doLayout();
		if(Pipeline.hideGOCheckBox.isSelected()){
			CyLayouts.getLayout("kamada-kawai").doLayout();
		}else {
			CyLayouts.getLayout("jgraph-tree").doLayout();
		}

	}
}
