package cytoscape.PFPPipeline.GOParser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import org.apache.commons.io.IOUtils;

import cytoscape.PFPPipeline.Pipeline;
import cytoscape.PFPPipeline.GOParser.model.GoException;
import cytoscape.PFPPipeline.GOParser.model.GoParser.OntologyDefineType;
import cytoscape.PFPPipeline.GOParser.model.OntologyGraphModel;
import cytoscape.PFPPipeline.GOParser.term.Constants;
import cytoscape.PFPPipeline.GOParser.term.GO_term;

public class SearchOntology {
	
	enum OpType {
		GETSIMILARITY, GETPATHWAY, EXIT
	};
	
	public static ArrayList<ArrayList> interactorPaths;
	public static ArrayList<ArrayList> paths;
	public static ArrayList<String> path;
	public static SearchOntology myOntologyobj;

	DecimalFormat df;

	String idPatternStr = null;

	String idPrefixStr = null;

	String connectStr = null;

	OntologyDefineType ontologyType;

	HashMap<String, GO_term> termMap;

	OntologyGraphModel graphModel;
	public SearchOntology(){
		
	}
	public SearchOntology(File ontologyFile, OntologyDefineType _ontologyType,
			String _idPatternStr, String _idPrefixStr, String _connectStr,
			DecimalFormat _df) {
		ontologyType = _ontologyType;
		idPatternStr = _idPatternStr;
		idPrefixStr = _idPrefixStr;
		connectStr = _connectStr;
		df = _df;
		long begin = System.currentTimeMillis();
		try {
			termMap = OntologyGraphModel.loadGoDefinition(ontologyFile,
					idPatternStr, idPrefixStr, connectStr, ontologyType);
			graphModel = new OntologyGraphModel(termMap);
		} catch (GoException e) {
			e.printStackTrace();
			System.out.println("Fail to create graph model");
			System.exit(-1);
		}
		long end = System.currentTimeMillis();
		// System.out.println("It took " + (end - begin) +
		// " ms to load the ontology " + ontologyFile.getName());

	}
	
	// GO Id to Name conversion.
	public static String id2Name(String id){		
		GO_term term = myOntologyobj.transformString2Term(id);		
		return term.getName();
		
	}
	
	@SuppressWarnings("unchecked")
	public static void getPaths() throws IOException {
		DecimalFormat df = new DecimalFormat("0000000");
		String idPatternStr, idPrefixStr, connectStr;
		OntologyDefineType ontologyType = OntologyDefineType.UNKNOWN;
		idPatternStr = Constants.geneOntologyIdPatternStr;
		connectStr = Constants.geneOntologyOboConnector;
		idPrefixStr = Constants.geneOntologyPrefix;
		ontologyType = OntologyDefineType.OBO;
		idPatternStr = Constants.geneOntologyIdPatternStr;
//		JOptionPane.showMessageDialog(null, "Path GO: "+ new SearchOntology().getClass().getClassLoader().getResource("gene_ontology_ext.obo").getPath());
//		String goDBPath=new SearchOntology().getClass().getClassLoader().getResource("gene_ontology_ext.obo").getPath();
//		File f = new File(goDBPath);
//		if(f.exists()) { 
//			JOptionPane.showMessageDialog(null,"path exists");
//		}
//		System.out.println("plu2:"+goDBPath);//gene_ontology_ext.obo
//		ontologyFile = new File(goDBPath);
		URL path=Pipeline.class.getResource("/gene_ontology_ext.obo");
		File ontologyFile=null;		
		try {
			ontologyFile = File.createTempFile("ontology", ".obo");
			ontologyFile.deleteOnExit();
			FileOutputStream out = new FileOutputStream(ontologyFile);
			IOUtils.copy(new SearchOntology().getClass().getClassLoader().getResourceAsStream("gene_ontology_ext.obo"), out);
			
		} catch (IOException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
		}
		myOntologyobj = new SearchOntology(ontologyFile, ontologyType,
				idPatternStr, idPrefixStr, connectStr, df);
		
		interactorPaths= new ArrayList<ArrayList>();
		
		for (int i=0; i<Pipeline.filteredInteractors.size();i++){
			paths = new ArrayList<ArrayList>();
//			System.out.print("\nTotal annotations: "+Pipeline.goAnnotations[i].size());
			if(Pipeline.goAnnotations[i].size() > 0){
				//Interactor i has annotations
				for (int j=0;j<Pipeline.goAnnotations[i].size(); j++){
					//System.out.println("\nGetting annotations for "+Pipeline.goAnnotations[i].get(j)+": ");
					GO_term term = myOntologyobj.transformString2Term(Pipeline.goAnnotations[i].get(j));
					myOntologyobj.getPathway(term);					
				}				
			}
			interactorPaths.add(paths);
		}
//		ArrayList<String> mypath;
//		for(int h=0;h<interactorPaths.size();h++){
//			System.out.print("\nPaths for "+(h+1)+" interactor: ");
//		for(int i=0;i<interactorPaths.get(h).size();i++){
//			mypath= (ArrayList<String>) interactorPaths.get(h).get(i);
//			System.out.print("\n");
//			
//			for (int j=0;j<mypath.size(); j++){
//				System.out.print(mypath.get(j)+ " ");
//			}			
//		}
//		}	

	}
	
	void getPathway(GO_term term) {		
		if (graphModel == null || term == null) {
			return;
		}
		Object[][] objs = graphModel.getPathsOfNode2Root(term);
		//System.out.print("\nThe pathways between " + term.getId()
			//	+ " and the root(0003674) are:");
		int k = 0;
		for (int i = 0; i < objs.length; i++) {

			if (objs[i][objs[i].length - 1].toString().contains("0003674")) {
				k++;
				String node;
				path = new ArrayList<String>();
				//System.out.print("\nPath " + (k) + ":\t");
				for (int j = 0; j < objs[i].length; j++) {
					node=((GO_term) objs[i][j]).getId();
					//System.out.print( node + " ");
					path.add(node);
				}
				paths.add(path);
			}
		}
	}

	/************************************************/

	public void pathsBetweenNodes(GO_term term1, GO_term term2) {
		if (term1 == null || term2 == null) {
			return;
		}
		HashMap map = graphModel.getPathsBetweenNodes(term1, term2);

	}

	GO_term[] getGetTerm(int termNum, BufferedReader in,
			ArrayList<String> idNameList) throws IOException {
		ArrayList<GO_term> termList = new ArrayList<GO_term>();
		while (termList.size() < termNum) {
			System.out
					.println(termNum
							+ " node names are necessary to calculate semantic similarity");
			System.out.println("Please input " + (termNum - termList.size())
					+ " term name(s) again");
			String[] array = in.readLine().split("[, ]+");
			for (int i = 0; i < array.length; i++)
				idNameList.add(array[i]);
			for (int i = 0; i < idNameList.size(); i++) {
				GO_term term = transformString2Term(idNameList.get(i));
				if (term != null)
					termList.add(term);
			}
			idNameList.clear();
		}
		return termList.toArray(new GO_term[0]);
	}

	public GO_term transformString2Term(String idName) {
		Pattern p = Pattern.compile(idPatternStr);
		Matcher m = p.matcher(idName);
		if (m.find()) {
			// idName = df.format(m.group());
			return termMap.get(idName);
		} else {
			return null;
		}
	}

	void getSimilarity(GO_term term1, GO_term term2, byte methodType) {
		if (graphModel == null || term1 == null || term2 == null) {
			return;
		}
		double value = graphModel.evalSimilarity(term1, term2, methodType);
		System.out.println("The semantic similarity between ");
		System.out.print(term1.getId() + " " + term2.getId() + " is: " + value);
	}



	void getPathway(GO_term term1, GO_term term2) {
		if (term1 == null || term2 == null) {
			return;
		}
		HashMap map = graphModel.getPathsBetweenNodes(term1, term2);
		HashSet commonSet = (HashSet) map.keySet().toArray()[0];
		Object[] commomTerms = commonSet.toArray();
		System.out.println("The common ancestors between " + term1.getId()
				+ " and " + term2.getId() + " are:\n\t");
		for (int i = 0; i < commomTerms.length; i++) {
			System.out.print((((GO_term) commomTerms[i])).getId() + " ");
		}
		System.out.println("\nthe commonPathways are:");
		Vector<Object> values = (Vector<Object>) map.get(commonSet);
		for (int i = 0; i < values.size(); i++) {
			System.out.print("\nPath " + (i + 1) + ":\t");
			Object[][] path = (Object[][]) values.get(i);
			for (int j = 0; j < path[i].length; j++) {
				System.out.print(path[i][j]);
			}
		}
	}

	static void printUsage() {
		System.out
				.println("Usage: java edu.hust.go.example.ConsoleExample -option");
		System.out
				.println("-ontologyFile ./data/gene_ontology.obo.2005-06-01, necessary!");
		System.out
				.println("-ontologyType OBO, the value can be OBO, OBOXML, RDFXML or OWL, default value is OBO");
		System.out
				.println("-getSim, calcualte the similarity between two ontology nodes.");
		System.out
				.println("\tthus it should be followed by two term parameter");
		System.out
				.println("-getPathway, infer the pathway between the target node to root node.");
		System.out
				.println("\tthus it should be followed by one term parameter");
		System.out
				.println("-term 0001, the specified term ID, it can appear more than 1 times.");
	}
}
