package cytoscape.PFPPipeline;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.SwingConstants;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import cytoscape.Cytoscape;
import cytoscape.PFPPipeline.Interactions.InteractorProtein;
import cytoscape.PFPPipeline.GOParser.SearchOntology;
import cytoscape.plugin.CytoscapePlugin;
import cytoscape.task.Task;
import cytoscape.task.TaskMonitor;
import cytoscape.task.ui.JTaskConfig;
import cytoscape.task.util.TaskManager;
import cytoscape.view.cytopanels.CytoPanelImp;
/**
 * A protein Annotation plugin that uses heterogeneous data with GO based
 * similarity to find the function of uncharacterized proteins.
 * 
 */
public class Pipeline extends CytoscapePlugin {

	public Interactions interactionsClassObject;
	public Motifs motifClassObject;
	public static String query;
	
	public CytoPanelImp ctrlPanel;
	public pluginPanel pluginPanel;
		
	public Font customizedFont;
	public JLabel inputProteinLabel;
	public JButton searchInteractorsButton;
	public HintTextField inputUniProtField;
	public JButton calculateScoreButton;
	public JLabel calculateScoreLabel;
	public JButton filterNetworkButton;
	public JLabel filterNetworkLabel;
	public HintTextField filterNetworkField;
	public JButton showPredictionsButton;
	public JLabel inputGOLabel;
	public HintTextField inputGOField;
	public JButton showUniqueGOButton;
	public JButton startOverButton;
	public JButton reFilterButton;
	
	public JTextPane infoMessagePane;
	
	public JCheckBox hideInteractorsCheckBox;
	public static JCheckBox hideGOCheckBox;
	public JCheckBox showNamesCheckBox;

	public BuildNetwork buildNetworkObject;
	public static Double PIScore[];
	public static List<Double> upPIScore; // PIS score after filtering
	public static List<InteractorProtein> filteredInteractors;
	public static ArrayList<String>[] goAnnotations; // Annotations of each
														// interactor protein.
	public static Integer step=0;
	public static ArrayList<String> targetAnnotations;
	public static ArrayList<Double> targetAnnotationsSim;
	public static Double simThreshold = 2.0;
	public static Double goSimThreshold = 0.0;
	public static ArrayList<ArrayList> upInteractorPaths; // Interactor Paths above GO Threshold.
	public static boolean showNamesFlag=false;
	private String stepOneText,stepTwoText,stepThreeText;
	private boolean goFlag=false;
//	private AttributedString stepThreeText;
	
	public Pipeline() {
		
		// (2) Create a JPanel object
		pluginPanel = new pluginPanel();
		
		pluginPanel.setToolTipText("Protein Function Prediction-Pipeline Plugin Panel");

		inputProteinLabel = new JLabel("Input Protein ID(UniProt): ", 10);
		customizedFont = new Font(inputProteinLabel.getFont().getName(), Font.BOLD,
				inputProteinLabel.getFont().getSize());
		inputProteinLabel.setFont(customizedFont);
		inputUniProtField = new HintTextField("Input UniProt ID...");
		inputUniProtField.setToolTipText("Input the protein id to be annotated.");
		searchInteractorsButton = new JButton("Search Interactors");
		searchInteractorsButton.setFont(customizedFont);
		searchInteractorsButton.setToolTipText("Click to retrieve interactions from IntACT and DIP Databases.");
		
		calculateScoreLabel= new JLabel("Calculate PIS Scores among Interactor Proteins");
		calculateScoreLabel.setFont(customizedFont);
		
		calculateScoreButton= new JButton("Calculate Protein Interaction Scores");
		calculateScoreButton.setFont(customizedFont);
		calculateScoreButton.setToolTipText("Click to calculate interaction scores among proteins based upon their  sequence similarity and shared conserved Motifs");

		filterNetworkLabel = new JLabel(
				"Input Similarity Threshold(PIS Score) between (0-2): ", 10);
		filterNetworkLabel.setFont(customizedFont);
		filterNetworkField = new HintTextField("e.g., 0.5");
		filterNetworkButton = new JButton("Filter Network");
		filterNetworkButton.setToolTipText("Click to filter the protein interaction network");
		filterNetworkButton.setFont(customizedFont);

		inputGOLabel = new JLabel("Input GO Similarity Threshold between (0-1): ", 10);
		inputGOLabel.setFont(customizedFont);
		inputGOField = new HintTextField("e.g., 0.8");
		showPredictionsButton = new JButton("Calculate and Show Predictions in Gene Ontology");
		showPredictionsButton.setFont(customizedFont);
		showPredictionsButton.setToolTipText("Click to see predicted terms for query protein");

		startOverButton = new JButton("Start Over");
		startOverButton.setFont(customizedFont);
		startOverButton.setToolTipText("Click to restart the prediction process.");

		
		hideInteractorsCheckBox= new JCheckBox("Hide Under Scored Interactors");
		hideGOCheckBox= new JCheckBox("Hide GO Nodes");
		showNamesCheckBox= new JCheckBox("Show Names instead of IDs.        ");
		
		reFilterButton = new JButton("Refilter Interaction Network");
		reFilterButton.setFont(customizedFont);
		reFilterButton.setToolTipText("Click to try with a different threshold value of PIS.");
		
		showUniqueGOButton=new JButton("Show Unique Predicted Functions");
		showUniqueGOButton.setFont(customizedFont);
		showUniqueGOButton.setToolTipText("Shows Unique set of Predicted Functions for query protein.");

		motifClassObject = new Motifs();
		interactionsClassObject = new Interactions();		
		upInteractorPaths= new ArrayList<ArrayList>();
		
		infoMessagePane= new JTextPane();
		stepOneText="The PFP(Protein Function Prediction)-Pipeline consists of three main steps: \n\n1- Given a UniProt ID, the first step retrieves the interaction information for a query protein form IntAct and DIP databases."
				+ "\n\n2- The second step calculates the PIS(Protein Interaction Scores) among the interacting proteins. "
				+ "The interactors crossing the given threshold are considered in step 3 for calculating GO(Gene Ontology) based Similarity."
				+ "\n\n3- In step 3, GO terms crossing the given similarity threshold are considered as target (predicted) annotations.";
		
		stepTwoText="In a Protein Interaction Network (PPIN) a Protein Interaction Score (PIS) between two interacting proteins uP (query protein in this case) and iPj (interactor protein) is calculated as follows,"
				+ "\n\nPIS(uP,iPj) = MS(uP,iPj ) + SS(uP,iPj), ∀iPj∈PPIN"
				+ "\n\nWhere MS(uP,iPj) measures the motif similarity of the two proteins, and SS(uP, iPj) measures the sequence similarity of the two proteins."
				+ "\n\n1- We define the motif similarity score between uP and iPj as the number of common motifs conserved between the two interacting proteins (dividend of the following equation) normalized to the minimum number of motifs obtained for the two proteins in isolation (divisor of the following equation.)"+ 
				"\n\nMS(Pi,Pj) =|motif (uP ) ∩ motif (iPj )| /\n\t\t\t\t\t\t\tmin( |motif(uP )|, |motif(iPj)| ), ∀iPj ∈ PPIN"
				+ "\n\n2- To calculate sequence similarity among proteins We use a normalized BLAST score, defined as the BLAST score of the two proteins divided by the self score of the query (i.e., the BLAST score of the protein against itself), as defined in the following equations:"
				+ "\n\nSS(uP, iPj) =BLAST (uP, iPj) / BLAST(uP), ∀iPj∈PPIN"
				+ "\n\nInteracting nodes with high PIS are more likely to correspond to reliable interactions and therefore identify proteins that actually participate in common functions.";
		
		stepThreeText="To infer more precise molecular activity of the query protein."
				+ " We calculate GO similarity score (GOSS) among all the candidate functions of interacting proteins based on GO. GOSS is calculated as follows, "
				+ "\n\nGOSS( Hi(iPj), Hz(iPk) )=| Hi(iPj) ∩ Hz(iPk) | /\n\t\t\t\t\t\t\t\t\t\t\t\t\t\t  min( |Hi(iPj)|, |Hz(iPk)| )"
				+ "\n\nWhere Hi(iPj) is a GO Hierarchy of annotation i for interacting protein iPj, thus | Hi(iPj) ∩ Hz(iPk) | is the overlap count between the two hierarchies, and  min( |Hi(iPj)|, |Hz(iPk)| ) gives the minimum length between the two hierarchies.";
		
		//		stepThreeText= new AttributedString("GOSS(Hi,Hj)=|Hi,Hj|/min(|Hi|,|Hj|)");
//		stepThreeText.addAttribute(TextAttribute.SUPERSCRIPT, TextAttribute.SUPERSCRIPT_SUPER, 6, 6);
		infoMessagePane.setText(stepOneText);
		infoMessagePane.setOpaque(false);
		infoMessagePane.setBackground(new Color(0,0,0,0));
		infoMessagePane.setBorder(new CompoundBorder(new EmptyBorder(10, 10, 10, 10), new LineBorder(new Color(0,0,0,0))));
		
		StyledDocument doc = infoMessagePane.getStyledDocument();
		SimpleAttributeSet center = new SimpleAttributeSet();
		StyleConstants.setAlignment(center, StyleConstants.ALIGN_JUSTIFIED );
		doc.setParagraphAttributes(0, doc.getLength(), center, false);
		infoMessagePane.setPreferredSize(new Dimension(350,530));
		buildNetworkObject = new BuildNetwork();

		// Three steps

		// (1) Get the handler to cytoPanel west
		ctrlPanel = (CytoPanelImp) Cytoscape.getDesktop().getCytoPanel(
				SwingConstants.WEST);

		
		pluginPanel.add(showNamesCheckBox);
		pluginPanel.add(inputProteinLabel);
		pluginPanel.add(inputUniProtField);
		// pluginPanel.add("\n");
		pluginPanel.add(searchInteractorsButton);
//		infoMessagePane.setText(stepThreeText);
		pluginPanel.add(infoMessagePane);
	

		// *********************** STEP 01 **********************

		searchInteractorsButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {				
				
				// TODO Auto-generated method stub
				
				if (inputUniProtField.getText().isEmpty()) {
					JOptionPane
							.showMessageDialog(null,"Please Input Uniprot ID of protien.");
					
				} 
				else {
					if (inputUniProtField.getText().length()!=6) {
						JOptionPane.showMessageDialog(null,"Please Input a valid Uniprot ID of protien. e.g. P02557");					
						}
					else{
						step = 1;
						query=inputUniProtField.getText();
//						 Create Interaction Task.
						Task task = new ProteinInteractionTask();

						// Configure JTask
						JTaskConfig config = new JTaskConfig();
						config.displayCancelButton(true);
						config.displayCloseButton(true);
						config.setModal(true); 
						config.setOwner(Cytoscape.getDesktop());
						config.displayStatus(true);
						config.setAutoDispose(false);

						/*
						 * Execute Task via TaskManager. This automatically pops-open a
						 * JTask Dialog Box. This method will block until the JTask
						 * Dialog Box is disposed.
						 */
						boolean success = TaskManager.executeTask(task, config);

						if (success) {
							
							if(Interactions.interactorProteins.size()>0){								

							// Hide previous buttons
							inputProteinLabel.setVisible(false);
							inputUniProtField.setVisible(false);
							searchInteractorsButton.setVisible(false);
							infoMessagePane.setVisible(false);
							pluginPanel.remove(infoMessagePane); 
							inputUniProtField.setText("");

							// Display after the interaction network is built.
							pluginPanel.add(calculateScoreButton);
							infoMessagePane.setText(stepTwoText);
							pluginPanel.add(infoMessagePane);
							infoMessagePane.setVisible(true);
							if(!calculateScoreButton.isVisible()){
								calculateScoreButton.setVisible(true);
							}
						}
						else{
							JOptionPane.showMessageDialog(null,"No protein protein interaction information could be found for "+query+" protein");
						}					

						}						
					}
				}
				
				//Test for Q9NQS7
//				query = inputUniProtField.getText();
//				query = "P06169";
//				query = "P06243"; //Yeast protein
//				query="P06208";
//				String currentDir = System.getProperty("user.dir");
//			    System.out.println("Current dir using System:" +currentDir);
//				query = "P23443";
//				String goDBPath=Pipeline.this.getClass().getClassLoader().getResource("gene_ontology_ext.obo").getPath();
//				JOptionPane.showMessageDialog(null,"path value: "+goDBPath);
//				File f = new File(goDBPath);
//				if(f.exists()) { 
//					JOptionPane.showMessageDialog(null,"path exists");
//				}
				
				

			}
		});
		/*
		 * Action for Calculate Score button
		 * */
		calculateScoreButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				// TODO Auto-generated method stub
				Task task = new PISScoreTask();

				// Configure JTask
				JTaskConfig config = new JTaskConfig();
				config.displayCancelButton(true);
				config.displayCloseButton(true);
				config.setOwner(Cytoscape.getDesktop());
				config.setModal(true); //?
				config.displayStatus(true);
				config.setAutoDispose(false);

				boolean success = TaskManager.executeTask(task, config);

				if (success) {
					// Hide previous buttons					
					calculateScoreButton.setVisible(false);
					infoMessagePane.setVisible(false);
					pluginPanel.remove(infoMessagePane); 
					pluginPanel.add(filterNetworkLabel);
					pluginPanel.add(filterNetworkField);
					// pluginPanel.add("\n");
					pluginPanel.add(filterNetworkButton);
					pluginPanel.add(infoMessagePane);
					infoMessagePane.setVisible(true);

					if (!filterNetworkLabel.isVisible()) {
						filterNetworkLabel.setVisible(true);
						filterNetworkField.setVisible(true);
						filterNetworkButton.setVisible(true);
					}
									

				}		
				
				/*
				pluginPanel.add(filterNetworkLabel);
				pluginPanel.add(filterNetworkField);
				// pluginPanel.add("\n");
				pluginPanel.add(filterNetworkButton);

				if (!filterNetworkLabel.isVisible()) {
					filterNetworkLabel.setVisible(true);
					filterNetworkField.setVisible(true);
					filterNetworkButton.setVisible(true);
				}*/
				
			}});
		
		/*
		 * 
		 * Step 02: Action to GO similarity Button!!
		 */
		filterNetworkButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				// TODO Auto-generated method stub
				if (filterNetworkField.getText().isEmpty()) {
					JOptionPane
							.showMessageDialog(null,
									"Input Similarity Threshold(PIS Score) between (0-2).");

				} else {
					simThreshold = Double.parseDouble(filterNetworkField.getText());
					if (simThreshold >= 0 && simThreshold < 2) {
						
						Task task = new GOPathsTask();

						// Configure JTask
						JTaskConfig config = new JTaskConfig();
						config.displayCancelButton(true);
						config.displayCloseButton(true);
						config.setOwner(Cytoscape.getDesktop());
						config.setModal(true); //?
						config.displayStatus(true);
						config.setAutoDispose(false);

						// Show Cancel/Close Buttons
						// config.displayUserButtons(true);

						boolean success = TaskManager.executeTask(task, config);
//						System.out.println("Task return status! " + success);
						if(success && filteredInteractors.size()>=1 && goFlag==true){
							goFlag=false;
							infoMessagePane.setVisible(false);
							pluginPanel.remove(infoMessagePane); 
							filterNetworkField.setText("");
							filterNetworkLabel.setVisible(false);
							filterNetworkField.setVisible(false);
							filterNetworkButton.setVisible(false);
							if(filteredInteractors.size()==1){
								// Add showPredictionsButton (ShowPredictions).								
								pluginPanel.add(showPredictionsButton);
								infoMessagePane.setPreferredSize(new Dimension(350,350));
								infoMessagePane.setText(stepThreeText);
								pluginPanel.add(infoMessagePane);
								infoMessagePane.setVisible(true);
								if (!showPredictionsButton.isVisible()) {
									showPredictionsButton.setVisible(true);
									
								}
							}
							else{
								// Add showPredictionsButton (Show Predictions).
								pluginPanel.add(inputGOLabel);
								pluginPanel.add(inputGOField);
								pluginPanel.add(showPredictionsButton);
								pluginPanel.add(hideInteractorsCheckBox);
								pluginPanel.add(hideGOCheckBox);
								infoMessagePane.setPreferredSize(new Dimension(350,350));
								infoMessagePane.setText(stepThreeText);
								pluginPanel.add(infoMessagePane);
								infoMessagePane.setVisible(true);
								if (!showPredictionsButton.isVisible()) {
									inputGOLabel.setVisible(true);
									inputGOField.setVisible(true);
									showPredictionsButton.setVisible(true);
									hideInteractorsCheckBox.setVisible(true);
									hideGOCheckBox.setVisible(true);

								}}						
						}
						
					}else{
						JOptionPane.showMessageDialog(null, "Please Input PIS threshold between 0 and 2.");
					}
				}

			}
		});
		
		//CheckBox Listeners
		//Hide low scoring interactors box.
		hideInteractorsCheckBox.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {

				Task task = new HideOtherInteractorsTask();

				// Configure JTask
				JTaskConfig config = new JTaskConfig();
				config.displayCancelButton(true);
				config.displayCloseButton(true);
				config.setOwner(Cytoscape.getDesktop());
				config.setModal(true); //?
				config.displayStatus(true);
				config.setAutoDispose(false);				
				boolean success = TaskManager.executeTask(task, config);					
				
			
}});
		
		//Hide Go Nodes box.
		hideGOCheckBox.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {

				Task task = new HideGOTask();

				// Configure JTask
				JTaskConfig config = new JTaskConfig();
				config.displayCancelButton(true);
				config.displayCloseButton(true);
				config.setOwner(Cytoscape.getDesktop());
				config.setModal(true); //?
				config.displayStatus(true);
				config.setAutoDispose(false);

				boolean success = TaskManager.executeTask(task, config);					
				
			
			}});
		
		/* 
		 * Show Names Checkbox: Shows names when checked.
		 * */
				showNamesCheckBox.addActionListener(new ActionListener() {

					@Override
					public void actionPerformed(ActionEvent e) {
						
						if(showNamesCheckBox.isSelected()){
							showNamesFlag=true;	
						}
						else{
							showNamesFlag=false;
						}
						if(step>0){
							Task task = new ShowNamesTask();

							// Configure JTask
							JTaskConfig config = new JTaskConfig();
							config.displayCancelButton(true);
							config.displayCloseButton(true);
							config.setOwner(Cytoscape.getDesktop());
							config.setModal(true); //?
							config.displayStatus(true);
							config.setAutoDispose(false);
							boolean success = TaskManager.executeTask(task, config);
							
						}					
					
					}});
		
		/*
		 * 
		 * Button 3 Action Listener
		 */

		showPredictionsButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				// TODO Auto-generated method stub
				
				// JOptionPane.showMessageDialog(null, "Button 3!");
				// To show Predictions
				if(pluginPanel.isAncestorOf(inputGOField)){
					
					if (inputGOField.getText().isEmpty()) {
						JOptionPane
								.showMessageDialog(null,
										"Input GO Similarity Threshold between 0 and 1.");
					} else {
						goSimThreshold = Double.parseDouble(inputGOField
								.getText());
						if (goSimThreshold >= 0 && goSimThreshold < 1) {
							pluginPanel.remove(infoMessagePane);
							pluginPanel.add(showUniqueGOButton);
							pluginPanel.add(reFilterButton);
							pluginPanel.add(startOverButton);
							pluginPanel.add(infoMessagePane);
							Task task = new GOSimilarityTask();

							// Configure JTask
							JTaskConfig config = new JTaskConfig();
							config.displayCancelButton(true);
							config.displayCloseButton(true);
							config.setModal(true); 
							config.setOwner(Cytoscape.getDesktop());
							config.displayStatus(true);
							config.setAutoDispose(false);
							boolean success = TaskManager.executeTask(task, config);

							if (success) {
								showUniqueGOButton.setVisible(true);
								reFilterButton.setVisible(true);
								startOverButton.setVisible(true);
								
//								if (!startOverButton.isVisible()) {
//									startOverButton.setVisible(true);
//								}
								
							}
							
						} else {
							JOptionPane
									.showMessageDialog(null,
											"Input GO Similarity Threshold between 0 and 1.");
						}									

				}
					
				}else{
					/*Only one interactor.*/
					pluginPanel.remove(infoMessagePane);
					pluginPanel.add(showUniqueGOButton);
					pluginPanel.add(reFilterButton);
					pluginPanel.add(startOverButton);
					pluginPanel.add(infoMessagePane);
					Task task = new GOSimilarityTask();

					// Configure JTask
					JTaskConfig config = new JTaskConfig();
					config.displayCancelButton(true);
					config.displayCloseButton(true);
					config.setModal(true); 
					config.setOwner(Cytoscape.getDesktop());
					config.displayStatus(true);
					config.setAutoDispose(false);

					boolean success = TaskManager.executeTask(task, config);

					if (success) {
						showUniqueGOButton.setVisible(true);
						reFilterButton.setVisible(true);
						startOverButton.setVisible(true);						
						
					}
					
				}
					
					

			}
		});
		/* Shows Unique GO target functions */
		showUniqueGOButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				if(filteredInteractors.size()==1){
					if(goAnnotations[0].size()>0){						
						targetAnnotations = new ArrayList<String>(goAnnotations[0]);
						

						Task task = new TargetTermsTask();
						// Configure JTask
						JTaskConfig config = new JTaskConfig();
						config.displayCancelButton(true);
						config.displayCloseButton(true);
						config.setModal(true); 
						config.setOwner(Cytoscape.getDesktop());
						config.displayStatus(true);
						config.setAutoDispose(false);

						boolean success = TaskManager.executeTask(task, config);				
						
					}else{
						JOptionPane.showMessageDialog(null, "No annotation could be predicted for "+Pipeline.query+" protein,\n above the given GO simialirty threshold. Change the threshold\n value and retry.");
					}
					
				}
				else if(filteredInteractors.size()>1 && targetAnnotations.size()>0){
					Task task = new TargetTermsTask();

					// Configure JTask
					JTaskConfig config = new JTaskConfig();
					config.displayCancelButton(true);
					config.displayCloseButton(true);
					config.setModal(true); 
					config.setOwner(Cytoscape.getDesktop());
					config.displayStatus(true);
					config.setAutoDispose(false);

					boolean success = TaskManager.executeTask(task, config);
					
				}
				else{
					JOptionPane.showMessageDialog(null, "No annotation could be predicted for "+Pipeline.query+" protein,\n above the given GO simialirty threshold. Change the threshold\n value and retry.");
				}	
				
				
				
			}});
		/*Button to try with different PIS Score.*/
		reFilterButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				// Show Interaction Network
				simThreshold=2.1; //set similarity to max.
				
				step=2;
				buildNetworkObject.createNetwork();
//				buildNetworkObject.showFileredNetwork(buildNetworkObject.cyInteractionScoreNetwork);
				inputGOField.setText("");				
				reFilterButton.setVisible(false);
				showUniqueGOButton.setVisible(false);
				infoMessagePane.setVisible(false);
				pluginPanel.remove(infoMessagePane); 
				showPredictionsButton.setVisible(false);
				if (filteredInteractors.size() > 1) {
					inputGOLabel.setVisible(false);
					inputGOField.setVisible(false);
					pluginPanel.remove(inputGOField);
				}				
				startOverButton.setVisible(false);
				filterNetworkLabel.setVisible(true);
				filterNetworkField.setVisible(true);
				filterNetworkButton.setVisible(true);
				infoMessagePane.setPreferredSize(new Dimension(350,530));
				infoMessagePane.setText(stepTwoText);
				pluginPanel.add(infoMessagePane);
				infoMessagePane.setVisible(true);
			}
		});
		
		/*
		 * 
		 * Button 4 Action Listener
		 */

		startOverButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				// TODO Auto-generated method stub
				inputGOField.setText("");

				infoMessagePane.setVisible(false);
				pluginPanel.remove(infoMessagePane);
				showPredictionsButton.setVisible(false);
				showUniqueGOButton.setVisible(false);
				if (filteredInteractors.size() > 1) {
					inputGOLabel.setVisible(false);
					inputGOField.setVisible(false);
					pluginPanel.remove(inputGOField);
				}
				//Reset threshold to maximum.
				simThreshold=2.1;
				reFilterButton.setVisible(false);
				startOverButton.setVisible(false);
				inputProteinLabel.setVisible(true);
				inputUniProtField.setVisible(true);
				searchInteractorsButton.setVisible(true);
				infoMessagePane.setPreferredSize(new Dimension(350,530));
				infoMessagePane.setText(stepOneText);
				pluginPanel.add(infoMessagePane);
				infoMessagePane.setVisible(true);
			}
		});

		

		// (3) Add the JPanel object to the cytoPanel west
		ctrlPanel.add("PFP-Pipeline Panel", pluginPanel);

		// Select the panel after the plugin is initialized
		// WISDOM PANEL
		int indexInCytoPanel = ctrlPanel.indexOfComponent("PFP-Pipeline Panel");
		ctrlPanel.setSelectedIndex(indexInCytoPanel);
	}

	/*
	 * 
	 * Calculates GO similarity among different annotation paths.
	 */
	@SuppressWarnings("unchecked")
	public void calculateGOSimilarity(Double goSIMThreshold) {
		ArrayList<String> singlePath, interactorPath, sameList;
		Double goSIM = 0.0;
		String term;
		targetAnnotations = new ArrayList<String>();
		targetAnnotationsSim = new ArrayList<Double>();
		upInteractorPaths= new ArrayList<ArrayList>();
		

		for (int i = 0; i < filteredInteractors.size(); i++) {

			for (int j = 0; j < SearchOntology.interactorPaths.get(i).size(); j++) {
				// Single path(j) of interactor i.
				singlePath = (ArrayList<String>) SearchOntology.interactorPaths
						.get(i).get(j);

				// Compare with the paths of all other interactors.
				for (int k = i + 1; k < filteredInteractors.size(); k++) {
					for (int l = 0; l < SearchOntology.interactorPaths.get(k)
							.size(); l++) {
						// System.out.println("The value of k and l is: "+i+" : "+j+" : "+k+" : "+l);
						interactorPath = (ArrayList<String>) SearchOntology.interactorPaths
								.get(k).get(l);

						// Calculate GO similarity.
						if (interactorPath.size() > 0) {
							if (singlePath.size() > interactorPath.size()) {
								sameList = new ArrayList<String>(singlePath);
								sameList.retainAll(interactorPath);
								goSIM = ((double) (sameList.size() - 1) / (interactorPath
										.size() - 1));
								// System.out.println("Same List: "+sameList.toString());
								// System.out.println("interactorPath: "+interactorPath.toString());
								if (goSIM >= goSIMThreshold) {
									term = interactorPath.get(0);
									upInteractorPaths.add(interactorPath);
									
									if (!targetAnnotations.contains(term)) {
										targetAnnotations.add(term);
										targetAnnotationsSim.add(goSIM);
										
									}

								}

							} else {
								sameList = new ArrayList<String>(singlePath);
								sameList.retainAll(interactorPath);
								goSIM = ((double) (sameList.size() - 1) / (singlePath
										.size() - 1));
								// System.out.println(goSIM+" Same List2: "+sameList.toString());
								// System.out.println("interactorPath2: "+interactorPath.toString());
								if (goSIM >= goSIMThreshold) {
									term = singlePath.get(0);
									upInteractorPaths.add(singlePath);
									if (!targetAnnotations.contains(term)) {
										targetAnnotations.add(term);
										targetAnnotationsSim.add(goSIM);
										
									}

								}
							}
						}

					}
				}

			}

		}
		
	}

	/*
	 * 
	 * Filter interaction network above given threshold.
	 */

	public void filterNetwork(Double threshold) {
		filteredInteractors = new ArrayList<InteractorProtein>();
		Iterator<InteractorProtein> iterator = interactionsClassObject.interactorProteins.iterator();
		upPIScore = new ArrayList<Double>();
		int j = 0;
		InteractorProtein temp;
		while (iterator.hasNext()) {
			temp = iterator.next();
			
			if (PIScore[j] >= threshold) {
				filteredInteractors.add(temp);
				upPIScore.add(PIScore[j]);
			}
			j++;
		}

	}

	class pluginPanel extends JPanel {
		public pluginPanel() {
		}
	}

	class HintTextField extends JTextField implements FocusListener {

		private final String hint;

		public HintTextField(final String hint) {
			super(hint, 10);
			this.hint = hint;
			super.addFocusListener(this);
		}

		@Override
		public void focusGained(FocusEvent e) {
			if (this.getText().isEmpty()) {
				super.setText("");
			}
		}

		@Override
		public void focusLost(FocusEvent e) {
			if (this.getText().isEmpty()) {
				super.setText(hint);
			}
		}

		@Override
		public String getText() {
			String typed = super.getText();
			return typed.equals(hint) ? "" : typed;
		}
	}

	// TASK CLASSES
	/**
	 * Protein Interaction Task class, used to retrieve interaction information from DIP and IntAct Databases.
	 */
	public class ProteinInteractionTask implements Task {
		private TaskMonitor taskMonitor = null;
		private boolean interrupted = false;
		private int exceptionIndex = Integer.MAX_VALUE;

		/**
		 * Constructor.
		 * 
		 * @param max
		 *            Max Count Value.
		 * @param countDelay
		 *            Delay between each count (in milliseconds).
		 */
		public ProteinInteractionTask() {
		}

		/**
		 * Run the Task.
		 */
		public void run() {
			// taskMonitor.setStatus("Loading...");
			if (taskMonitor == null) {
				throw new IllegalStateException("Task Monitor is not set.");
			}

			// TODO Auto-generated method stub
			
				interactionsClassObject = new Interactions();

				if (taskMonitor != null) {
					taskMonitor
							.setStatus("Searching for Interactions from DIP and IntAct Databases...");
					interactionsClassObject.searchInteractions(query);					
					taskMonitor.setStatus("Please wait creating network...");
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
					buildNetworkObject.createNetwork();
					if ( interrupted == true ){
						   return;  // exit gracefully
					 }
					taskMonitor.setStatus("Interation Networks Retrieved");
					taskMonitor.setPercentCompleted(100);
					taskMonitor.setEstimatedTimeRemaining(0);
				}			
		}

		/**
		 * Non-blocking call to interrupt the task.
		 */
		public void halt() {
			this.interrupted = true;
		}

		/**
		 * Sets the Task Monitor.
		 * 
		 * @param taskMonitor
		 *            TaskMonitor Object.
		 */
		public void setTaskMonitor(TaskMonitor taskMonitor) {
			if (this.taskMonitor != null) {
				throw new IllegalStateException("Task Monitor is already set.");
			}
			this.taskMonitor = taskMonitor;
		}

		/**
		 * Gets the Task Title.
		 * 
		 * @return human readable task title.
		 */
		public String getTitle() {
			return new String("Retrieving Protein Interaction Information...");
		}
	}

	/**
	 * Sample Task, used to illustrate the Task Framework. This tasks counts
	 * from 0 to maxValue.
	 */
	public class PISScoreTask implements Task {
		private TaskMonitor taskMonitor = null;
		private boolean interrupted = false;
		private int exceptionIndex = Integer.MAX_VALUE;

		/**
		 * Constructor.
		 * 
		 * @param max
		 *            Max Count Value.
		 * @param countDelay
		 *            Delay between each count (in milliseconds).
		 */
		public PISScoreTask() {
		}

		/**
		 * Run the Task.
		 */
		public void run() {
			// taskMonitor.setStatus("Loading...");
			if (taskMonitor == null) {
				throw new IllegalStateException("Task Monitor is not set.");
			}

			// TODO Auto-generated method stub
			try {
				float bar = 0.0f, progress;
				progress = (float)(100.0 / Interactions.interactorProteins.size());
				
				boolean flag = false;
				InteractorProtein it1;
				List<String> interactorMotifs;
				List<String> sameList;

				float motSim, iSim, jSim, seqSim = 0.0f;
				PIScore = new Double[Interactions.interactorProteins.size()];
				

				List<String> queryMotifs = new ArrayList<String>(
						motifClassObject.searchMotifs(query));
				String status="";

				if (!queryMotifs.isEmpty()) {
					flag = true;
				} else {
					status="Note: The Query protien ("+query+") has no Motifs Conserved!\n";
				}
				iSim = BlastWrapper.getBlastScore(query, query);
				Iterator<InteractorProtein> it = interactionsClassObject.interactorProteins.iterator();
				
				int i = 0;
				int count = 0;
				while (it.hasNext()) {
					 if ( interrupted == true ){
						   return;  // exit gracefully
					 }
					it1 = it.next();
//					System.out.println(i + "-------->" + it1.getUniprotID());
					motSim = 0.0f;
					// Flag true to calculate motif similarity.
					if (flag == true) {
						interactorMotifs = new ArrayList<String>(
								motifClassObject.searchMotifs(it1.getUniprotID()));
						// interactorMotifs=motifClassObject.searchMotifs(it.next());
						if (interactorMotifs.size() > 0) {
							if (queryMotifs.size() > interactorMotifs.size()) {
								sameList = new ArrayList<String>(queryMotifs);
								sameList.retainAll(interactorMotifs);
								motSim = ((float) sameList.size())
										/ interactorMotifs.size();
							} else {
								sameList = new ArrayList<String>(queryMotifs);
								sameList.retainAll(interactorMotifs);
								motSim = ((float) sameList.size())
										/ queryMotifs.size();
								// System.out.println(interactorMotifs);
								// System.out.println(it1+" "+similarity+" SameList: "+sameList.size()+"ineractorr motifs: "+interactorMotifs.size());

							}
						}
					}
					jSim = BlastWrapper.getBlastScore(query, it1.getUniprotID());
					if (iSim > 0.0) {
						seqSim = jSim / iSim;
					}
					// totSim[i]=0.0f;
					DecimalFormat dFormat = new DecimalFormat("#.####");
					PIScore[i] = (Double.parseDouble(dFormat.format((seqSim + motSim)))) ;					
					it1.setPISValue(PIScore[i]);
//					System.out.println(it1.getUniprotID() + " ======= " + motSim + "  +  "+ seqSim + " = " + PIScore[i]);
					i++;
					count++;
					bar = bar + progress;
					int percent = ((int)Math.floor((double)bar))-1;
					if(percent<1){
						percent=0;
					}
//					System.out.println("percent: " + percent+" bar: "+bar+" progress: "+progress);
					
						if (taskMonitor != null) {
							taskMonitor.setPercentCompleted(percent);
							
							taskMonitor.setStatus(status+"Calculating PIS Scores [" + percent+ "% completed.]");
							
							taskMonitor.setEstimatedTimeRemaining(100 - percent);
						}

					try {
						Thread.sleep(1000);
					} // end try
					catch (InterruptedException ex) {
					}
				}
			} catch (IOException ex) {
			}
			
			Pipeline.step=2;
			taskMonitor.setStatus("Please wait while creating Network...");
			buildNetworkObject.createNetwork();
			if (taskMonitor != null) {
				if (taskMonitor != null) {
					taskMonitor.setPercentCompleted(100);
					taskMonitor.setStatus("Completed");
					taskMonitor.setEstimatedTimeRemaining(0);
				}
			}
		}

		/**
		 * Non-blocking call to interrupt the task.
		 */
		public void halt() {
			this.interrupted = true;
		}

		/**
		 * Sets the Task Monitor.
		 * 
		 * @param taskMonitor
		 *            TaskMonitor Object.
		 */
		public void setTaskMonitor(TaskMonitor taskMonitor) {
			if (this.taskMonitor != null) {
				throw new IllegalStateException("Task Monitor is already set.");
			}
			this.taskMonitor = taskMonitor;
		}

		/**
		 * Gets the Task Title.
		 * 
		 * @return human readable task title.
		 */
		public String getTitle() {
			return new String("Calculating Protein Interaction Scores...");
		}
	}
	
	/* 
	 * This Task fetches GO annotations for interacting proteins and searches
	 * for GO paths to calculate GO similarity.
	 */
	public class GOPathsTask implements Task {
		private TaskMonitor taskMonitor = null;
		private boolean interrupted = false;
		private int exceptionIndex = Integer.MAX_VALUE;

		/**
		 * Constructor.
		 * 
		 * @param max
		 *            Max Count Value.
		 * @param countDelay
		 *            Delay between each count (in milliseconds).
		 */
		public GOPathsTask() {
		}

		/**
		 * Run the Task.
		 */
		public void run() {
			// taskMonitor.setStatus("Loading...");
			if (taskMonitor == null) {
				throw new IllegalStateException("Task Monitor is not set.");
			}

			// TODO Auto-generated method stub
			if (taskMonitor != null) {
//				System.out.println("SIM239: "+Pipeline.simThreshold);
				filterNetwork(simThreshold);			

					// Go to gene ontology part if filtered network
					// contains more than one node.
					if (filteredInteractors.size() > 1) {
						Pipeline.step=3;
						taskMonitor.setStatus("Retrieving function annotation information for Interacting proteins");
						taskMonitor.setPercentCompleted(3);
						// Search for functions of each node in the
						// filtered network.
						goAnnotations = new ArrayList[filteredInteractors
								.size()];
						for (int i = 0; i < filteredInteractors.size(); i++) {
							goAnnotations[i] = new ArrayList<String>(
									UniProtQueryTool
											.getAnnotations(filteredInteractors
													.get(i).getUniprotID()));
							if(goAnnotations[i].size()>0 && goFlag==false){
								goFlag=true;
							}
//							System.out.println("In size > 1 " + i + " : "+ goAnnotations[i].size());
						}
						if(goFlag==true){
							// Get all paths of all annotations.
							try {
								taskMonitor.setStatus("Loading Gene Ontology...");
								taskMonitor.setPercentCompleted(23);
								SearchOntology.getPaths();
								taskMonitor.setStatus("GO Paths Retrieved.");
								taskMonitor.setPercentCompleted(49);
							} catch (IOException e1) {
								// TODO Auto-generated catch block
								e1.printStackTrace();
							}
//							System.out.println("upPIScore: " + upPIScore.size());
							// Create network of all nodes.
							taskMonitor.setStatus("Creating Network...");
							taskMonitor.setPercentCompleted(60);
							buildNetworkObject.createNetwork();						

							taskMonitor.setStatus("Task Finished!");
							taskMonitor.setPercentCompleted(100);
						}else{
							taskMonitor.setStatus("No Functional annotation is found for the interacting proteins under the given PIS value! :(");
							taskMonitor.setPercentCompleted(100);
							
						}
						

					} else if (filteredInteractors.size() == 1) {
						Pipeline.step=3;
						// Find functions of unique interactor.
						goAnnotations = new ArrayList[filteredInteractors
								.size()];
						goAnnotations[0] = new ArrayList<String>(
								UniProtQueryTool
										.getAnnotations(filteredInteractors
												.get(0).getUniprotID()));
						if(goAnnotations[0].size()>0 && goFlag==false){
							goFlag=true;
						}
//						System.out.println("In size==1"	+ goAnnotations[0].size());
						if(goFlag==true){
							// Get all paths of all annotations.
							try {
								taskMonitor.setStatus("Loading Gene Ontology...");
								taskMonitor.setPercentCompleted(23);
								SearchOntology.getPaths();
								taskMonitor.setStatus("GO Paths Retrieved.");
								taskMonitor.setPercentCompleted(49);
							} catch (IOException e1) {
								// TODO Auto-generated catch block
								e1.printStackTrace();
							}
							for (int j = 0; j < SearchOntology.interactorPaths.get(0).size(); j++) {
								// Single path(j) of interactor i.
								ArrayList<String> singlePath = (ArrayList<String>) SearchOntology.interactorPaths
										.get(0).get(j);
								if(filteredInteractors.size()==1){
									upInteractorPaths.add(singlePath);
								}
							}
							// Create network of all nodes.
							taskMonitor.setStatus("Creating Network...");
							taskMonitor.setPercentCompleted(60);
							buildNetworkObject.createNetwork(); 
							
							taskMonitor.setStatus("Task Finished!");
							taskMonitor.setPercentCompleted(100);
						}else{
							taskMonitor.setStatus("No Functional annotation is found for the interacting proteins under the given PIS value! :(");
							taskMonitor.setPercentCompleted(100);
							
						}
						

					} else { // No potential interator found.
						taskMonitor.setStatus("No interactor crosses the threshold value specified! :(");
						try {
							Thread.sleep(1000);
						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
						taskMonitor.setPercentCompleted(100);
						
					}
				}			
		}

		/**
		 * Non-blocking call to interrupt the task.
		 */
		public void halt() {
			this.interrupted = true;
		}

		/**
		 * Sets the Task Monitor.
		 * 
		 * @param taskMonitor
		 *            TaskMonitor Object.
		 */
		public void setTaskMonitor(TaskMonitor taskMonitor) {
			if (this.taskMonitor != null) {
				throw new IllegalStateException("Task Monitor is already set.");
			}
			this.taskMonitor = taskMonitor;
		}

		/**
		 * Gets the Task Title.
		 * 
		 * @return human readable task title.
		 */
		public String getTitle() {
			return new String("GO Path Searching Task");
		}
	}
	/* 
	 * Reloads the new network view by hiding GO paths.
	 */
	public class HideOtherInteractorsTask implements Task {
		private TaskMonitor taskMonitor = null;
		private boolean interrupted = false;
		private int exceptionIndex = Integer.MAX_VALUE;

		/**
		 * Constructor.
		 * 
		 * @param max
		 *            Max Count Value.
		 * @param countDelay
		 *            Delay between each count (in milliseconds).
		 */
		public HideOtherInteractorsTask() {
			
		}

		/**
		 * Run the Task.
		 */
		public void run() {
			// taskMonitor.setStatus("Loading...");
			if (taskMonitor == null) {
				throw new IllegalStateException("Task Monitor is not set.");
			}

			// TODO Auto-generated method stub
			if (taskMonitor != null) {
				taskMonitor.setStatus("Creating network please wait...");
				taskMonitor.setPercentCompleted(13);
				if (hideInteractorsCheckBox.isSelected() && hideGOCheckBox.isSelected()) {
					buildNetworkObject.showFileredNetwork(buildNetworkObject.cyUpInteractionNetwork);
					taskMonitor.setStatus("Done");
					taskMonitor.setPercentCompleted(100);

				} else if ((!hideInteractorsCheckBox.isSelected()) && hideGOCheckBox.isSelected()) {
					buildNetworkObject.showFileredNetwork(buildNetworkObject.cyInteractionScoreNetwork);
					taskMonitor.setStatus("Done");
					taskMonitor.setPercentCompleted(100);
				}else if (hideInteractorsCheckBox.isSelected() && (!hideGOCheckBox.isSelected())) {
					buildNetworkObject.showFileredNetwork(buildNetworkObject.cyOnlyGONodesNetwork);
					taskMonitor.setStatus("Done");
					taskMonitor.setPercentCompleted(100);
				}else if ((!hideInteractorsCheckBox.isSelected()) && (!hideGOCheckBox.isSelected())) {
					buildNetworkObject.showFileredNetwork(buildNetworkObject.cyGOAllNodesNetwork);
					taskMonitor.setStatus("Done");
					taskMonitor.setPercentCompleted(100);
				}

			}		
		}

		/**
		 * Non-blocking call to interrupt the task.
		 */
		public void halt() {
			this.interrupted = true;
		}

		/**
		 * Sets the Task Monitor.
		 * 
		 * @param taskMonitor
		 *            TaskMonitor Object.
		 */
		public void setTaskMonitor(TaskMonitor taskMonitor) {
			if (this.taskMonitor != null) {
				throw new IllegalStateException("Task Monitor is already set.");
			}
			this.taskMonitor = taskMonitor;
		}

		/**
		 * Gets the Task Title.
		 * 
		 * @return human readable task title.
		 */
		public String getTitle() {
			return new String("Creating Selected Network");
		}
	}
	
	/* 
	 * Reloads the new network view by hiding GO paths.
	 */
	public class HideGOTask implements Task {
		private TaskMonitor taskMonitor = null;
		private boolean interrupted = false;
		private int exceptionIndex = Integer.MAX_VALUE;

		/**
		 * Constructor.
		 * 
		 * @param max
		 *            Max Count Value.
		 * @param countDelay
		 *            Delay between each count (in milliseconds).
		 */
		public HideGOTask() {
		}

		/**
		 * Run the Task.
		 */
		public void run() {
			// taskMonitor.setStatus("Loading...");
			if (taskMonitor == null) {
				throw new IllegalStateException("Task Monitor is not set.");
			}

			// TODO Auto-generated method stub
			if (taskMonitor != null) {
				taskMonitor.setStatus("Creating network please wait...");
				taskMonitor.setPercentCompleted(13);
				if (hideInteractorsCheckBox.isSelected() && hideGOCheckBox.isSelected()) {
					buildNetworkObject.showFileredNetwork(buildNetworkObject.cyUpInteractionNetwork);
					taskMonitor.setStatus("Done");
					taskMonitor.setPercentCompleted(100);

				} else if ((!hideInteractorsCheckBox.isSelected()) && hideGOCheckBox.isSelected()) {
					buildNetworkObject.showFileredNetwork(buildNetworkObject.cyInteractionScoreNetwork);
					taskMonitor.setStatus("Done");
					taskMonitor.setPercentCompleted(100);
				}else if (hideInteractorsCheckBox.isSelected() && (!hideGOCheckBox.isSelected())) {
					buildNetworkObject.showFileredNetwork(buildNetworkObject.cyOnlyGONodesNetwork);
					taskMonitor.setStatus("Done");
					taskMonitor.setPercentCompleted(100);
				}else if ((!hideInteractorsCheckBox.isSelected()) && (!hideGOCheckBox.isSelected())) {
					buildNetworkObject.showFileredNetwork(buildNetworkObject.cyGOAllNodesNetwork);
					taskMonitor.setStatus("Done");
					taskMonitor.setPercentCompleted(100);
				}

			}		
		}

		/**
		 * Non-blocking call to interrupt the task.
		 */
		public void halt() {
			this.interrupted = true;
		}

		/**
		 * Sets the Task Monitor.
		 * 
		 * @param taskMonitor
		 *            TaskMonitor Object.
		 */
		public void setTaskMonitor(TaskMonitor taskMonitor) {
			if (this.taskMonitor != null) {
				throw new IllegalStateException("Task Monitor is already set.");
			}
			this.taskMonitor = taskMonitor;
		}

		/**
		 * Gets the Task Title.
		 * 
		 * @return human readable task title.
		 */
		public String getTitle() {
			return new String("Hide GO Nodes Task");
		}
	}
	
	/* Create GO Network based no similarity. */
	
	public class GOSimilarityTask implements Task {
		private TaskMonitor taskMonitor = null;
		private boolean interrupted = false;

		/**
		 * Constructor.
		 * 		
		 */
		public GOSimilarityTask() {
		}

		/**
		 * Run the Task.
		 */
		public void run() {
			// taskMonitor.setStatus("Loading...");
			if (taskMonitor == null) {
				throw new IllegalStateException("Task Monitor is not set.");
			}

			// TODO Auto-generated method stub
			if (taskMonitor != null) {
				if(filteredInteractors.size()==1){
					hideInteractorsCheckBox.setSelected(false);
					hideGOCheckBox.setSelected(false);					
					hideInteractorsCheckBox.setVisible(false);
					hideGOCheckBox.setVisible(false);
					hideGOCheckBox.setVisible(false);					
					step = 4;
					taskMonitor.setStatus("Creating Network please wait...");
					taskMonitor.setPercentCompleted(38);
					// Create Network
					buildNetworkObject.createNetwork();
					taskMonitor.setStatus("Done");
					taskMonitor.setPercentCompleted(100);
				}else if(filteredInteractors.size()>1){
					hideInteractorsCheckBox.setSelected(false);
					hideGOCheckBox.setSelected(false);					
					hideInteractorsCheckBox.setVisible(false);
					hideGOCheckBox.setVisible(false);
					hideGOCheckBox.setVisible(false);
					
					step = 4;
					taskMonitor.setStatus("Please wait while calculating GO Similarity...");
					taskMonitor.setPercentCompleted(5);
					// Calculate Similarity.
					calculateGOSimilarity(Pipeline.goSimThreshold);
					taskMonitor.setStatus("Creating Network please wait...");
					taskMonitor.setPercentCompleted(38);
					// Create Network
					buildNetworkObject.createNetwork();
					taskMonitor.setStatus("Done");
					taskMonitor.setPercentCompleted(100);
					}				
				
				}					
			}

		/**
		 * Non-blocking call to interrupt the task.
		 */
		public void halt() {
			this.interrupted = true;
		}

		/**
		 * Sets the Task Monitor.
		 * 
		 * @param taskMonitor
		 *            TaskMonitor Object.
		 */
		public void setTaskMonitor(TaskMonitor taskMonitor) {
			if (this.taskMonitor != null) {
				throw new IllegalStateException("Task Monitor is already set.");
			}
			this.taskMonitor = taskMonitor;
		}

		/**
		 * Gets the Task Title.
		 * 
		 * @return human readable task title.
		 */
		public String getTitle() {
			return new String("Gene Ontology based Similarity Task");
		}
	}
	
	/* Show Names/IDs Task */
	
	public class ShowNamesTask implements Task {
		private TaskMonitor taskMonitor = null;
		private boolean interrupted = false;

		/**
		 * Constructor.
		 * 		
		 */
		public ShowNamesTask() {
		}

		/**
		 * Run the Task.
		 */
		public void run() {
			 
			if (taskMonitor == null) {
				throw new IllegalStateException("Task Monitor is not set.");
			}

			// TODO Auto-generated method stub
			if (taskMonitor != null) {
				
				taskMonitor.setStatus("Please wait while refreshing view...");
				taskMonitor.setPercentCompleted(50);				
				taskMonitor.setStatus("Creating Network please wait...");
				// Create Network
				if(step==3){

					if (hideInteractorsCheckBox.isSelected() && hideGOCheckBox.isSelected()) {
						buildNetworkObject.showFileredNetwork(buildNetworkObject.cyUpInteractionNetwork);
						taskMonitor.setStatus("Done");
						taskMonitor.setPercentCompleted(100);

					} else if ((!hideInteractorsCheckBox.isSelected()) && hideGOCheckBox.isSelected()) {
						buildNetworkObject.showFileredNetwork(buildNetworkObject.cyInteractionScoreNetwork);
						taskMonitor.setStatus("Done");
						taskMonitor.setPercentCompleted(100);
					}else if (hideInteractorsCheckBox.isSelected() && (!hideGOCheckBox.isSelected())) {
						buildNetworkObject.showFileredNetwork(buildNetworkObject.cyOnlyGONodesNetwork);
						taskMonitor.setStatus("Done");
						taskMonitor.setPercentCompleted(100);
					}else if ((!hideInteractorsCheckBox.isSelected()) && (!hideGOCheckBox.isSelected())) {
						buildNetworkObject.showFileredNetwork(buildNetworkObject.cyGOAllNodesNetwork);
						taskMonitor.setStatus("Done");
						taskMonitor.setPercentCompleted(100);
					}
					
				}else{
					buildNetworkObject.createNetwork();
					taskMonitor.setStatus("Done");
					taskMonitor.setPercentCompleted(100);
				}
				
				
			}
			}

		/**
		 * Non-blocking call to interrupt the task.
		 */
		public void halt() {
			this.interrupted = true;
		}

		/**
		 * Sets the Task Monitor.
		 * 
		 * @param taskMonitor
		 *            TaskMonitor Object.
		 */
		public void setTaskMonitor(TaskMonitor taskMonitor) {
			if (this.taskMonitor != null) {
				throw new IllegalStateException("Task Monitor is already set.");
			}
			this.taskMonitor = taskMonitor;
		}

		/**
		 * Gets the Task Title.
		 * 
		 * @return human readable task title.
		 */
		public String getTitle() {
			return new String("Change Names/IDs Task");
		}
	}
	
	/* Target GO Terms Network Task */
	public class TargetTermsTask implements Task{

		private TaskMonitor taskMonitor = null;
		private boolean interrupted = false;

		/**
		 * Constructor.
		 * 		
		 */
		public TargetTermsTask() {
		}

		/**
		 * Run the Task.
		 */
		public void run() {
			 
			if (taskMonitor == null) {
				throw new IllegalStateException("Task Monitor is not set.");
			}
			step=5;

			// TODO Auto-generated method stub
			if (taskMonitor != null) {
				
				taskMonitor.setStatus("Please wait while refreshing view...");
				taskMonitor.setPercentCompleted(50);				
				taskMonitor.setStatus("Creating Network please wait...");
				// Create Network
				buildNetworkObject.createNetwork();
				taskMonitor.setStatus("Done");
				taskMonitor.setPercentCompleted(100);
				
			}
			}

		/**
		 * Non-blocking call to interrupt the task.
		 */
		public void halt() {
			this.interrupted = true;
		}

		/**
		 * Sets the Task Monitor.
		 * 
		 * @param taskMonitor
		 *            TaskMonitor Object.
		 */
		public void setTaskMonitor(TaskMonitor taskMonitor) {
			if (this.taskMonitor != null) {
				throw new IllegalStateException("Task Monitor is already set.");
			}
			this.taskMonitor = taskMonitor;
		}

		/**
		 * Gets the Task Title.
		 * 
		 * @return human readable task title.
		 */
		public String getTitle() {
			return new String("Target Terms View Task!");
		}
	
	}

}
