/**
 * 
 */
package cytoscape.PFPPipeline;

import java.awt.Color;
import java.util.ArrayList;

import cytoscape.CyEdge;
import cytoscape.CyNetwork;
import cytoscape.CyNode;
import cytoscape.Cytoscape;
import cytoscape.actions.ApplyVisualStyleAction;
import cytoscape.data.CyAttributes;
import cytoscape.data.Semantics;
import cytoscape.dialogs.IconPopupButton.ApplyIconAction;
import cytoscape.view.CyNetworkView;
import cytoscape.visual.Arrow;
import cytoscape.visual.ArrowShape;
import cytoscape.visual.CalculatorCatalog;
import cytoscape.visual.EdgeAppearance;
import cytoscape.visual.EdgeAppearanceCalculator;
import cytoscape.visual.GlobalAppearanceCalculator;
import cytoscape.visual.LineStyle;
import cytoscape.visual.NodeAppearance;
import cytoscape.visual.NodeAppearanceCalculator;
import cytoscape.visual.NodeShape;
import cytoscape.visual.VisualMappingManager;
import cytoscape.visual.VisualPropertyType;
import cytoscape.visual.VisualStyle;
import cytoscape.visual.calculators.BasicCalculator;
import cytoscape.visual.calculators.Calculator;
import cytoscape.visual.mappings.BoundaryRangeValues;
import cytoscape.visual.mappings.ContinuousMapping;
import cytoscape.visual.mappings.DiscreteMapping;
import cytoscape.visual.mappings.Interpolator;
import cytoscape.visual.mappings.LinearNumberToColorInterpolator;
import cytoscape.visual.mappings.LinearNumberToNumberInterpolator;
import cytoscape.visual.mappings.ObjectMapping;
import cytoscape.visual.mappings.PassThroughMapping;

/**
 * @author hafeez
 * 
 */
public class VisualizeNetwork {
	/*
	 * Class Members*
	 */

	public static final String vsName = "PFP-Pipeline Visual Style";
	private VisualStyle vs;
	public ArrayList<CyNode> nodes;
	public ArrayList<CyEdge> edges;	
	CyAttributes cyNodeAttrs, cyEdgeAttrs;

	/**
	 * 
	 */
	public VisualizeNetwork() {
		// TODO Auto-generated constructor stub		
	}

	/*
	 * 
	 * Member Functions
	 */
	/*
	 * Apply visual style
	 * */
	
	public void ApplyVisualStyle(CyNetwork network){
		
		CyNetworkView networkView = Cytoscape.getCurrentNetworkView();		

		// get the VisualMappingManager and CalculatorCatalog
		VisualMappingManager manager = Cytoscape.getVisualMappingManager();
		CalculatorCatalog catalog = manager.getCalculatorCatalog();

		// check to see if a visual style with this name already exists
		vs = catalog.getVisualStyle(vsName);
		if (vs == null) {
			// if not, create it and add it to the catalog
			vs = createVisualStyle(network, vsName);
			catalog.addVisualStyle(vs);
		}
		
		networkView.setVisualStyle(vs.getName()); // not strictly necessary

		// actually apply the visual style
		manager.setVisualStyle(vs);
		networkView.redrawGraph(true,true);		
		
	}

	/**
	 * Creates visual style for this WordCloud network.
	 * 
	 * @param network
	 *            - network to apply this visual style
	 * @param name
	 *            - name of the network, to be appended to attribute names
	 */
	public VisualStyle createVisualStyle(CyNetwork network, String name) {
		vs = new VisualStyle(vsName);
		GlobalAppearanceCalculator globalAppCalc = new GlobalAppearanceCalculator();
		globalAppCalc.setDefaultBackgroundColor(new Color(70,130,180));
		vs.setGlobalAppearanceCalculator(globalAppCalc);

		createEdgeAppearance(network, name);
		createNodeAppearance(network, name);

		return vs;
	}

	/**
	 * Create edge appearances for this WordCloud network. Specify edge
	 * thickness mapped to the inputted word co occurence probability.
	 * 
	 * @param network
	 *            - network to apply this visual style
	 * @param name
	 *            - name to be appended to each of the attribute names
	 */
	private void createEdgeAppearance(CyNetwork network, String name) {
		EdgeAppearanceCalculator edgeAppCalc = new EdgeAppearanceCalculator(
				vs.getDependency());

		// Set the default edge appearance
		EdgeAppearance edgeAppear = new EdgeAppearance();
		edgeAppear.set(VisualPropertyType.EDGE_COLOR, new Color(0, 0, 0));
		edgeAppear.set(VisualPropertyType.EDGE_LINE_STYLE, LineStyle.SOLID);
		edgeAppear.set(VisualPropertyType.EDGE_LABEL_COLOR, new Color(250, 250,
				250));
		edgeAppear.set(VisualPropertyType.EDGE_SRCARROW_COLOR , new Color(0, 0, 0));
		// Edge ARROW mapping.

		DiscreteMapping arrowMapping = new DiscreteMapping(
				ArrowShape.ARROW, ObjectMapping.EDGE_MAPPING);
		arrowMapping.setControllingAttributeName(Semantics.INTERACTION);
		arrowMapping.putMapValue("GO", ArrowShape.ARROW);
		Calculator arrowShapeCalculator = new BasicCalculator(
				"Edge Arrow Calculator", arrowMapping,
				VisualPropertyType.EDGE_SRCARROW_SHAPE);
		edgeAppCalc.setCalculator(arrowShapeCalculator);		
//		edgeAppear.set(VisualPropertyType.EDGE_TGTARROW_SHAPE , ArrowShape.ARROW);
		
		
		// edgeAppear.set(VisualPropertyType.EDGE_LABEL , "edgeid");
		edgeAppCalc.setDefaultAppearance(edgeAppear);

		// Continuous Mapping - set edge line thickness based on the probability
		// ratio
		ContinuousMapping continuousMapping_edgeWidth = new ContinuousMapping(
				1, ObjectMapping.EDGE_MAPPING);
		continuousMapping_edgeWidth.setControllingAttributeName("edgeid");
		Interpolator numTonum2 = new LinearNumberToNumberInterpolator();
		continuousMapping_edgeWidth.setInterpolator(numTonum2);

		Double under_width = 0.5;
		Double min_width = 1.0;
		Double max_width = 3.0;
		Double max_width2 = 4.0;
		Double over_width = 5.0;

		// Create boundary conditions
		BoundaryRangeValues bv4 = new BoundaryRangeValues(under_width,
				min_width, max_width);
		BoundaryRangeValues bv5 = new BoundaryRangeValues(max_width, max_width2,
				over_width);
		continuousMapping_edgeWidth.addPoint(0.5, bv4);
		continuousMapping_edgeWidth.addPoint(1.4, bv5);

		Calculator edgeWidthCalculator = new BasicCalculator(name + "edgesize",
				continuousMapping_edgeWidth, VisualPropertyType.EDGE_LINE_WIDTH);
		edgeAppCalc.setCalculator(edgeWidthCalculator);

		// Discrete Mapping of Edge Label.
		PassThroughMapping me = new PassThroughMapping("", "label");
		final Calculator calce = new BasicCalculator(name + "-"
				+ "EdgeLabelMapping", me, VisualPropertyType.EDGE_LABEL);
		edgeAppCalc.setCalculator(calce);

		vs.setEdgeAppearanceCalculator(edgeAppCalc);
	}

	/**
	 * Create node appearances for this WordCloud network. Specify node size and
	 * label size based on the word probability values.
	 * 
	 * @param network
	 *            - network to apply this visual style
	 * @param name
	 *            - name to be appended to each of the attribute names
	 */
	private void createNodeAppearance(CyNetwork network, String name) {
		NodeAppearanceCalculator nodeAppCalc = new NodeAppearanceCalculator(
				vs.getDependency());

		// set the default appearance
		NodeAppearance nodeAppear = new NodeAppearance();
//		nodeAppear.set(VisualPropertyType.NODE_FILL_COLOR,new Color(30, 30, 30));
		nodeAppear.set(VisualPropertyType.NODE_LABEL_COLOR, new Color(255, 255,
				255));
		nodeAppear.set(VisualPropertyType.NODE_BORDER_COLOR, new Color(255,
				255, 255));
		nodeAppear.set(VisualPropertyType.NODE_SHAPE, NodeShape.ELLIPSE);
		nodeAppear.set(VisualPropertyType.NODE_SIZE, new Double(35.0));
		nodeAppear.set(VisualPropertyType.NODE_LINE_WIDTH, new Double(2.0));
		nodeAppear.set(VisualPropertyType.NODE_LINE_STYLE, LineStyle.SOLID);
		nodeAppear.set(VisualPropertyType.NODE_FONT_SIZE, new Integer(8));
		nodeAppCalc.setDefaultAppearance(nodeAppear);

		Integer min, max;
		BoundaryRangeValues bv0, bv1,bv2;
		
		// Node BORDER COLOR mapping.

		DiscreteMapping borderColorMapping = new DiscreteMapping(
				new Color(255,255, 255), ObjectMapping.NODE_MAPPING);
		borderColorMapping.setControllingAttributeName("Type");		
		borderColorMapping.putMapValue("GO", new Color(0,0,0));
		Calculator borderColorCalculator = new BasicCalculator(
				"Border Color Calculator", borderColorMapping,
				VisualPropertyType.NODE_BORDER_COLOR);
		nodeAppCalc.setCalculator(borderColorCalculator);
		
		// Node ToolTip mapping.

		DiscreteMapping toolTipmapping = new DiscreteMapping(
				"ToolTip Text", ObjectMapping.NODE_MAPPING);
		toolTipmapping.setControllingAttributeName("Type");
		toolTipmapping.putMapValue("query", "This is query protein.");
		toolTipmapping.putMapValue("protein", "Interactor protein");
		toolTipmapping.putMapValue("GO", "Gene Ontology Node.");
		Calculator toolTipCalculator = new BasicCalculator(
				"ToolTip Calculator", toolTipmapping,
				VisualPropertyType.NODE_TOOLTIP);
		nodeAppCalc.setCalculator(toolTipCalculator);
		
		// Mapping for target annotations
		
//		DiscreteMapping targetcolormapping = new DiscreteMapping(
//				Color.BLACK, ObjectMapping.NODE_MAPPING);
//		targetcolormapping.setControllingAttributeName("target");
//		targetcolormapping.putMapValue("yes", new Color(128,0,0));
//		targetcolormapping.putMapValue("no", new Color(30, 30, 30));
//		Calculator targetcolorCalculator = new BasicCalculator(
//				"Target node Color Calculator", targetcolormapping,
//				VisualPropertyType.NODE_FILL_COLOR);
//		nodeAppCalc.setCalculator(targetcolorCalculator);
		
//		 Discrete mapping for NODE SHAPE i.e. Circle for protein and rectangle for GO term.

		DiscreteMapping nodeShapemapping = new DiscreteMapping(
				NodeShape.ELLIPSE, ObjectMapping.NODE_MAPPING);
		nodeShapemapping.setControllingAttributeName("Type");
		nodeShapemapping.putMapValue("query", NodeShape.ROUND_RECT);
		nodeShapemapping.putMapValue("protein", NodeShape.ELLIPSE);
		nodeShapemapping.putMapValue("GO", NodeShape.RECT);
		Calculator nodeShapeCalculator = new BasicCalculator(
				"Example Node Shape Calculator", nodeShapemapping,
				VisualPropertyType.NODE_SHAPE);
		nodeAppCalc.setCalculator(nodeShapeCalculator);

		// Continuous Mapping - set node size based on the probability value

		ContinuousMapping continuousMapping_size = new ContinuousMapping(35,
				ObjectMapping.NODE_MAPPING);
		continuousMapping_size.setControllingAttributeName("NSize");
		Interpolator numTonum = new LinearNumberToNumberInterpolator();
		continuousMapping_size.setInterpolator(numTonum);

		min = 55;
		max = 60;

		// Boundary Conditions
		bv0 = new BoundaryRangeValues(min, min, min);
		bv1 = new BoundaryRangeValues(max, max, max);

		continuousMapping_size.addPoint(new Integer(0), bv0);
		continuousMapping_size.addPoint(new Integer(2), bv1);

		Calculator nodeSizeCalculator = new BasicCalculator(name + "size2size",
				continuousMapping_size, VisualPropertyType.NODE_SIZE);
		nodeAppCalc.setCalculator(nodeSizeCalculator);

		// PASSTHROUGH MAPPING FOR NODE LABEL
		
		PassThroughMapping pm = new PassThroughMapping(new String(),
				"shownLabel");
		Calculator nlc = new BasicCalculator(name + "shownLabel", pm,
				VisualPropertyType.NODE_LABEL);
		nodeAppCalc.setCalculator(nlc);

		
		// Continuous Mapping - set NODE COLOR based upon their COLOR VALUE attribute.
		ContinuousMapping continuousColorMappinG = new ContinuousMapping(
				new Color(0,0,0), ObjectMapping.NODE_MAPPING);
		continuousColorMappinG.setControllingAttributeName("colorValue");
		Interpolator colorInterpolator = new LinearNumberToColorInterpolator();
		continuousColorMappinG.setInterpolator(colorInterpolator);
		
		Color underColor = new Color(128,0,0); // if <1
		Color equalColor = new Color(0,0,0); // if >=1 <2
		Color greaterColor = new Color(139,37,0);	// if >2<3
		Color higherColor = new Color(0,100,0);//if >=3

		// Boundary Conditions < = >
		bv0 = new BoundaryRangeValues(underColor, equalColor, equalColor);
		bv1 = new BoundaryRangeValues(equalColor, greaterColor, greaterColor);
		bv2 =new BoundaryRangeValues(greaterColor, higherColor, higherColor);
		
		continuousColorMappinG.addPoint(new Double(1.0), bv0);
		continuousColorMappinG.addPoint(new Double(2.0), bv1);
		continuousColorMappinG.addPoint(new Double(3.0), bv2);

		Calculator nodeColorCalculator = new BasicCalculator(name
				+ "nodeColorCalculator", continuousColorMappinG,
				VisualPropertyType.NODE_FILL_COLOR);
		nodeAppCalc.setCalculator(nodeColorCalculator);

		vs.setNodeAppearanceCalculator(nodeAppCalc);
	}

}
